@@ -35,13 +35,13 @@ install_python() {
 install_pyfury() {
   echo "Python version $(python -V), path $(which python)"
   "$ROOT"/ci/deploy.sh install_pyarrow
-  pip install Cython wheel "numpy<2.0.0" pytest
+  pip install Cython wheel pytest
   pushd "$ROOT/python"
   pip list
-  export PATH=~/bin:$PATH
   echo "Install pyfury"
   # Fix strange installed deps not found
-  pip install setuptools
+  pip install setuptools -U
+  bazel build //:cp_fury_so
   pip install -v -e .
   popd
 }
@@ -78,6 +78,12 @@ install_bazel() {
   fi
 }
 
+install_bazel_windows() {
+  choco install bazel --version=6.3.2 --force
+  VERSION=`bazel version`
+  echo "bazel version: $VERSION"
+}
+
 JDKS=(
 "zulu21.28.85-ca-jdk21.0.0-linux_x64"
 "zulu17.44.17-ca-crac-jdk17.0.8-linux_x64"
@@ -269,6 +275,12 @@ case $1 in
         exit $testcode
       fi
       echo "Executing fury python tests succeeds"
+      ENABLE_FURY_CYTHON_SERIALIZATION=0 pytest -v -s --durations=60 pyfury/tests
+      testcode=$?
+      if [[ $testcode -ne 0 ]]; then
+        exit $testcode
+      fi
+      echo "Executing fury python tests succeeds"
     ;;
     go)
       echo "Executing fury go tests for go"
