@@ -480,8 +480,23 @@ public class CodeGenerator {
     return sb;
   }
 
+  private static final WeakHashMap<Class<?>, Boolean> sourcePublicLevelAccessible =
+      new WeakHashMap<>();
+
+  public static Class<?> getSourcePublicAccessibleParentClass(Class<?> clz) {
+    while (!sourcePublicAccessible(clz)) {
+      clz = clz.getSuperclass();
+    }
+    return clz;
+  }
+
   /** Returns true if class is public accessible from source. */
   public static boolean sourcePublicAccessible(Class<?> clz) {
+    return sourcePublicLevelAccessible.computeIfAbsent(
+        clz, CodeGenerator::classSourcePublicAccessible);
+  }
+
+  private static boolean classSourcePublicAccessible(Class<?> clz) {
     if (clz.isPrimitive()) {
       return true;
     }
@@ -491,8 +506,16 @@ public class CodeGenerator {
     return sourcePkgLevelAccessible(clz);
   }
 
+  private static final WeakHashMap<Class<?>, Boolean> sourcePkgLevelAccessible =
+      new WeakHashMap<>();
+
   /** Returns true if class is package level accessible from source. */
   public static boolean sourcePkgLevelAccessible(Class<?> clz) {
+    return sourcePkgLevelAccessible.computeIfAbsent(
+        clz, CodeGenerator::classSourcePkgLevelAccessible);
+  }
+
+  private static boolean classSourcePkgLevelAccessible(Class<?> clz) {
     if (clz.isPrimitive()) {
       return true;
     }
