@@ -69,6 +69,7 @@ public final class FuryBuilder {
   boolean compressInt = true;
   public LongEncoding longEncoding = LongEncoding.SLI;
   boolean compressString = false;
+  Boolean writeNumUtf16BytesForUtf8Encoding;
   CompatibleMode compatibleMode = CompatibleMode.SCHEMA_CONSISTENT;
   boolean checkJdkClassSerializable = true;
   Class<? extends Serializer> defaultJDKStreamSerializerType = ObjectStreamSerializer.class;
@@ -83,6 +84,7 @@ public final class FuryBuilder {
   boolean suppressClassRegistrationWarnings = true;
   boolean deserializeNonexistentEnumValueAsNull = false;
   boolean serializeEnumByName = false;
+  int bufferSizeLimitBytes = 128 * 1024;
   MetaCompressor metaCompressor = new DeflaterMetaCompressor();
 
   public FuryBuilder() {}
@@ -184,6 +186,28 @@ public final class FuryBuilder {
     return this;
   }
 
+  /**
+   * Whether write num_bytes of utf16 for utf8 encoding. With this option enabled, fury will write
+   * the num_bytes of utf16 before write utf8 encoded data, so that the deserialization can create
+   * the appropriate utf16 array for store the data, thus save one copy.
+   */
+  public FuryBuilder withWriteNumUtf16BytesForUtf8Encoding(
+      boolean writeNumUtf16BytesForUtf8Encoding) {
+    this.writeNumUtf16BytesForUtf8Encoding = writeNumUtf16BytesForUtf8Encoding;
+    return this;
+  }
+
+  /**
+   * Sets the limit for Fury's internal buffer. If the buffer size exceeds this limit, it will be
+   * reset to this limit after every serialization and deserialization.
+   *
+   * <p>The default is 128k.
+   */
+  public FuryBuilder withBufferSizeLimitBytes(int bufferSizeLimitBytes) {
+    this.bufferSizeLimitBytes = bufferSizeLimitBytes;
+    return this;
+  }
+
   /**
    * Set classloader for fury to load classes, this classloader can't up updated. Fury will cache
    * the class meta data, if classloader can be updated, there may be class meta collision if
@@ -351,7 +375,9 @@ public final class FuryBuilder {
       }
     }
     if (language != Language.JAVA) {
-      stringRefIgnored = false;
+      stringRefIgnored = true;
+      longEncoding = LongEncoding.PVL;
+      compressInt = true;
     }
     if (ENABLE_CLASS_REGISTRATION_FORCIBLY) {
       if (!requireClassRegistration) {
@@ -367,6 +393,9 @@ public final class FuryBuilder {
           ObjectStreamSerializer.class,
           Serializer.class);
     }
+    if (writeNumUtf16BytesForUtf8Encoding == null) {
+      writeNumUtf16BytesForUtf8Encoding = language == Language.JAVA;
+    }
     if (compatibleMode == CompatibleMode.COMPATIBLE) {
       checkClassVersion = false;
       if (deserializeNonexistentClass == null) {
@@ -388,13 +417,16 @@ public final class FuryBuilder {
       if (deserializeNonexistentClass == null) {
         deserializeNonexistentClass = false;
       }
-      if (scopedMetaShareEnabled != null) {
+      if (scopedMetaShareEnabled != null && scopedMetaShareEnabled) {
         LOG.warn("Scoped meta share is for CompatibleMode only, disable it for {}", compatibleMode);
       }
       scopedMetaShareEnabled = false;
       if (metaShareEnabled == null) {
         metaShareEnabled = false;
       }
+      if (language != Language.JAVA) {
+        checkClassVersion = true;
+      }
     }
     if (!requireClassRegistration) {
       LOG.warn(
