@@ -237,10 +237,17 @@ public class ClassDef implements Serializable {
         Descriptor descriptor =
             descriptorsMap.get(fieldInfo.getDefinedClass() + "." + fieldInfo.getFieldName());
         Descriptor newDesc = fieldInfo.toDescriptor(resolver);
+        Class<?> rawType = newDesc.getRawType();
+        FieldType fieldType = fieldInfo.getFieldType();
+        if (fieldType instanceof RegisteredFieldType) {
+          String typeAlias = String.valueOf(((RegisteredFieldType) fieldType).getClassId());
+          if (!typeAlias.equals(newDesc.getTypeName())) {
+            newDesc = newDesc.copyWithTypeName(typeAlias);
+          }
+        }
         if (descriptor != null) {
           // Make DescriptorGrouper have consistent order whether field exist or not
           // fury builtin types skip
-          Class<?> rawType = newDesc.getRawType();
           if (rawType.isEnum()
               || rawType.isAssignableFrom(descriptor.getRawType())
               || NonexistentClass.isNonexistent(rawType)
@@ -464,7 +471,12 @@ public class ClassDef implements Serializable {
 
     @Override
     public TypeRef<?> toTypeToken(ClassResolver classResolver) {
-      return TypeRef.of(classResolver.getRegisteredClass(classId), new TypeExtMeta(trackingRef));
+      Class<?> cls = classResolver.getRegisteredClass(classId);
+      if (cls == null) {
+        LOG.warn("Class {} not registered, take it as Struct type for deserialization.", classId);
+        cls = NonexistentClass.NonexistentMetaShared.class;
+      }
+      return TypeRef.of(cls, new TypeExtMeta(trackingRef));
     }
 
     @Override
