@@ -19,16 +19,22 @@
 
 package org.apache.fury.resolver;
 
+import static org.apache.fury.Fury.NOT_SUPPORT_XLANG;
 import static org.apache.fury.meta.ClassDef.SIZE_TWO_BYTES_FLAG;
+import static org.apache.fury.meta.Encoders.GENERIC_ENCODER;
 import static org.apache.fury.meta.Encoders.PACKAGE_DECODER;
 import static org.apache.fury.meta.Encoders.PACKAGE_ENCODER;
 import static org.apache.fury.meta.Encoders.TYPE_NAME_DECODER;
+import static org.apache.fury.meta.Encoders.encodePackage;
+import static org.apache.fury.meta.Encoders.encodeTypeName;
 import static org.apache.fury.serializer.CodegenSerializer.loadCodegenSerializer;
 import static org.apache.fury.serializer.CodegenSerializer.loadCompatibleCodegenSerializer;
 import static org.apache.fury.serializer.CodegenSerializer.supportCodegenForJavaSerialization;
 import static org.apache.fury.type.TypeUtils.OBJECT_TYPE;
 import static org.apache.fury.type.TypeUtils.getRawType;
 
+import com.google.common.collect.BiMap;
+import com.google.common.collect.HashBiMap;
 import java.io.Externalizable;
 import java.io.IOException;
 import java.io.Serializable;
@@ -73,6 +79,7 @@ import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.atomic.AtomicReference;
+import java.util.function.Function;
 import java.util.function.Supplier;
 import java.util.stream.Collectors;
 import org.apache.fury.Fury;
@@ -132,7 +139,6 @@ import org.apache.fury.serializer.Serializer;
 import org.apache.fury.serializer.SerializerFactory;
 import org.apache.fury.serializer.Serializers;
 import org.apache.fury.serializer.StringSerializer;
-import org.apache.fury.serializer.StructSerializer;
 import org.apache.fury.serializer.TimeSerializers;
 import org.apache.fury.serializer.collection.ChildContainerSerializers;
 import org.apache.fury.serializer.collection.CollectionSerializer;
@@ -149,11 +155,14 @@ import org.apache.fury.serializer.scala.SingletonMapSerializer;
 import org.apache.fury.serializer.scala.SingletonObjectSerializer;
 import org.apache.fury.serializer.shim.ShimDispatcher;
 import org.apache.fury.type.Descriptor;
+import org.apache.fury.type.DescriptorGrouper;
 import org.apache.fury.type.GenericType;
 import org.apache.fury.type.ScalaTypes;
 import org.apache.fury.type.TypeUtils;
+import org.apache.fury.type.Types;
 import org.apache.fury.util.GraalvmSupport;
 import org.apache.fury.util.Preconditions;
+import org.apache.fury.util.StringUtils;
 import org.apache.fury.util.function.Functions;
 
 /**
@@ -161,7 +170,7 @@ import org.apache.fury.util.function.Functions;
  * up relations between serializer and types.
  */
 @SuppressWarnings({"rawtypes", "unchecked"})
-public class ClassResolver {
+public class ClassResolver implements TypeResolver {
   private static final Logger LOG = LoggerFactory.getLogger(ClassResolver.class);
 
   // bit 0 unset indicates class is written as an id.
@@ -207,29 +216,25 @@ public class ClassResolver {
   public static final short CLASS_CLASS_ID = (short) (PRIMITIVE_DOUBLE_ARRAY_CLASS_ID + 6);
   public static final short EMPTY_OBJECT_ID = (short) (PRIMITIVE_DOUBLE_ARRAY_CLASS_ID + 7);
   // use a lower load factor to minimize hash collision
-  private static final float loadFactor = 0.25f;
   private static final float furyMapLoadFactor = 0.25f;
   private static final int estimatedNumRegistered = 150;
   private static final String SET_META__CONTEXT_MSG =
       "Meta context must be set before serialization, "
           + "please set meta context by SerializationContext.setMetaContext";
-  private static final ClassInfo NIL_CLASS_INFO =
-      new ClassInfo(null, null, null, null, false, null, null, ClassResolver.NO_CLASS_ID);
+  static final ClassInfo NIL_CLASS_INFO =
+      new ClassInfo(null, null, null, null, false, null, NO_CLASS_ID, NOT_SUPPORT_XLANG);
 
   private final Fury fury;
+  XtypeResolver xtypeResolver;
   private ClassInfo[] registeredId2ClassInfo = new ClassInfo[] {};
 
   // IdentityMap has better lookup performance, when loadFactor is 0.05f, performance is better
   private final IdentityMap<Class<?>, ClassInfo> classInfoMap =
       new IdentityMap<>(estimatedNumRegistered, furyMapLoadFactor);
   private ClassInfo classInfoCache;
-  private final ObjectMap<MetaStringBytes, Class<?>> classNameBytes2Class =
-      new ObjectMap<>(16, furyMapLoadFactor);
   // Every deserialization for unregistered class will query it, performance is important.
-  private final ObjectMap<ClassNameBytes, ClassInfo> compositeClassNameBytes2ClassInfo =
+  private final ObjectMap<TypeNameBytes, ClassInfo> compositeNameBytes2ClassInfo =
       new ObjectMap<>(16, furyMapLoadFactor);
-  private final HashMap<Short, Class<?>> typeIdToClassXLangMap = new HashMap<>(8, loadFactor);
-  private final HashMap<String, Class<?>> typeTagToClassXLangMap = new HashMap<>(8, loadFactor);
   private final MetaStringResolver metaStringResolver;
   private final boolean metaContextShareEnabled;
   private final Map<Class<?>, ClassDef> classDefMap = new HashMap<>();
@@ -246,7 +251,8 @@ public class ClassResolver {
     private SerializerFactory serializerFactory;
     private final IdentityMap<Class<?>, Short> registeredClassIdMap =
         new IdentityMap<>(estimatedNumRegistered);
-    private final Map<String, Class<?>> registeredClasses = new HashMap<>(estimatedNumRegistered);
+    private final BiMap<String, Class<?>> registeredClasses =
+        HashBiMap.create(estimatedNumRegistered);
     // avoid potential recursive call for seq codec generation.
     // ex. A->field1: B, B.field1: A
     private final Set<Class<?>> getClassCtx = new HashSet<>();
@@ -260,7 +266,8 @@ public class ClassResolver {
         descriptorsCache = new ConcurrentHashMap<>();
     private ClassChecker classChecker = (classResolver, className) -> true;
     private GenericType objectGenericType;
-    private Map<List<ClassLoader>, CodeGenerator> codeGeneratorMap = new HashMap<>();
+    private final IdentityMap<Type, GenericType> genericTypes = new IdentityMap<>();
+    private final Map<List<ClassLoader>, CodeGenerator> codeGeneratorMap = new HashMap<>();
   }
 
   public ClassResolver(Fury fury) {
@@ -269,12 +276,13 @@ public class ClassResolver {
     classInfoCache = NIL_CLASS_INFO;
     metaContextShareEnabled = fury.getConfig().isMetaShareEnabled();
     extRegistry = new ExtRegistry();
-    extRegistry.objectGenericType = buildGenericType(OBJECT_TYPE);
     shimDispatcher = new ShimDispatcher(fury);
     ClassResolver._addGraalvmClassRegistry(fury.getConfig().getConfigHash(), this);
   }
 
+  @Override
   public void initialize() {
+    extRegistry.objectGenericType = buildGenericType(OBJECT_TYPE);
     register(LambdaSerializer.ReplaceStub.class, LAMBDA_STUB_ID);
     register(JdkProxySerializer.ReplaceStub.class, JDK_PROXY_STUB_ID);
     register(ReplaceResolveSerializer.ReplaceStub.class, REPLACE_STUB_ID);
@@ -433,18 +441,6 @@ public class ClassResolver {
     }
   }
 
-  /** register class with given type tag which will be used for cross-language serialization. */
-  public void register(Class<?> cls, String typeTag) {
-    if (fury.getLanguage() == Language.JAVA) {
-      throw new IllegalArgumentException(
-          "Java serialization should register class by "
-              + "Fury#register(Class) or Fury.register(Class<?>, Short)");
-    }
-    register(cls);
-    Preconditions.checkArgument(!typeTagToClassXLangMap.containsKey(typeTag));
-    addSerializer(cls, new StructSerializer<>(fury, cls, typeTag));
-  }
-
   /**
    * Register class with specified id. Currently class id must be `classId >= 0 && classId < 32767`.
    * In the future this limitation may be relaxed.
@@ -452,25 +448,8 @@ public class ClassResolver {
   public void register(Class<?> cls, int classId) {
     // class id must be less than Integer.MAX_VALUE/2 since we use bit 0 as class id flag.
     Preconditions.checkArgument(classId >= 0 && classId < Short.MAX_VALUE);
-    if (extRegistry.registeredClassIdMap.containsKey(cls)) {
-      throw new IllegalArgumentException(
-          String.format(
-              "Class %s already registered with id %s.",
-              cls, extRegistry.registeredClassIdMap.get(cls)));
-    }
-    if (extRegistry.registeredClasses.containsKey(cls.getName())) {
-      throw new IllegalArgumentException(
-          String.format(
-              "Class %s with name %s has been registered, registering class with same name are not allowed.",
-              extRegistry.registeredClasses.get(cls.getName()), cls.getName()));
-    }
     short id = (short) classId;
-    if (id < registeredId2ClassInfo.length && registeredId2ClassInfo[id] != null) {
-      throw new IllegalArgumentException(
-          String.format(
-              "Class %s with id %s has been registered, registering class %s with same id are not allowed.",
-              registeredId2ClassInfo[id].getCls(), id, cls.getName()));
-    }
+    checkRegistration(cls, id, cls.getName());
     extRegistry.registeredClassIdMap.put(cls, id);
     if (registeredId2ClassInfo.length <= id) {
       ClassInfo[] tmp = new ClassInfo[(id + 1) * 2];
@@ -481,7 +460,7 @@ public class ClassResolver {
     if (classInfo != null) {
       classInfo.classId = id;
     } else {
-      classInfo = new ClassInfo(this, cls, null, null, id);
+      classInfo = new ClassInfo(this, cls, null, id, NOT_SUPPORT_XLANG);
       // make `extRegistry.registeredClassIdMap` and `classInfoMap` share same classInfo
       // instances.
       classInfoMap.put(cls, classInfo);
@@ -496,7 +475,7 @@ public class ClassResolver {
     register(loadClass(className, false, 0, false), classId);
   }
 
-  public void register(Class<?> cls, Short id, boolean createSerializer) {
+  public void register(Class<?> cls, int id, boolean createSerializer) {
     register(cls, id);
     if (createSerializer) {
       createSerializerAhead(cls);
@@ -507,7 +486,90 @@ public class ClassResolver {
     register(loadClass(className, false, 0, false), classId, createSerializer);
   }
 
+  /**
+   * Register class with specified namespace and name. If a simpler namespace or type name is
+   * registered, the serialized class will have smaller payload size. In many cases, it type name
+   * has no conflict, namespace can be left as empty.
+   */
+  public void register(Class<?> cls, String namespace, String name) {
+    Preconditions.checkArgument(!Functions.isLambda(cls));
+    Preconditions.checkArgument(!ReflectionUtils.isJdkProxy(cls));
+    Preconditions.checkArgument(!cls.isArray());
+    String fullname = name;
+    if (namespace == null) {
+      namespace = "";
+    }
+    if (!StringUtils.isBlank(namespace)) {
+      fullname = namespace + "." + name;
+    }
+    checkRegistration(cls, (short) -1, fullname);
+    MetaStringBytes fullNameBytes =
+        metaStringResolver.getOrCreateMetaStringBytes(
+            GENERIC_ENCODER.encode(fullname, MetaString.Encoding.UTF_8));
+    MetaStringBytes nsBytes =
+        metaStringResolver.getOrCreateMetaStringBytes(encodePackage(namespace));
+    MetaStringBytes nameBytes = metaStringResolver.getOrCreateMetaStringBytes(encodeTypeName(name));
+    ClassInfo classInfo =
+        new ClassInfo(cls, fullNameBytes, nsBytes, nameBytes, false, null, NO_CLASS_ID, (short) -1);
+    classInfoMap.put(cls, classInfo);
+    compositeNameBytes2ClassInfo.put(
+        new TypeNameBytes(nsBytes.hashCode, nameBytes.hashCode), classInfo);
+    extRegistry.registeredClasses.put(fullname, cls);
+  }
+
+  private void checkRegistration(Class<?> cls, short classId, String name) {
+    if (extRegistry.registeredClassIdMap.containsKey(cls)) {
+      throw new IllegalArgumentException(
+          String.format(
+              "Class %s already registered with id %s.",
+              cls, extRegistry.registeredClassIdMap.get(cls)));
+    }
+    if (classId > 0
+        && classId < registeredId2ClassInfo.length
+        && registeredId2ClassInfo[classId] != null) {
+      throw new IllegalArgumentException(
+          String.format(
+              "Class %s with id %s has been registered, registering class %s with same id are not allowed.",
+              registeredId2ClassInfo[classId].getCls(), classId, cls.getName()));
+    }
+    if (extRegistry.registeredClasses.containsKey(name)
+        || extRegistry.registeredClasses.inverse().containsKey(cls)) {
+      throw new IllegalArgumentException(
+          String.format(
+              "Class %s with name %s has been registered, registering class %s with same name are not allowed.",
+              extRegistry.registeredClasses.get(name), name, cls));
+    }
+  }
+
+  @Override
   public boolean isRegistered(Class<?> cls) {
+    return extRegistry.registeredClassIdMap.containsKey(cls)
+        || extRegistry.registeredClasses.inverse().containsKey(cls);
+  }
+
+  public boolean isRegisteredByName(String name) {
+    return extRegistry.registeredClasses.containsKey(name);
+  }
+
+  public boolean isRegisteredByName(Class<?> cls) {
+    return extRegistry.registeredClasses.inverse().containsKey(cls);
+  }
+
+  public String getRegisteredName(Class<?> cls) {
+    return extRegistry.registeredClasses.inverse().get(cls);
+  }
+
+  public Tuple2<String, String> getRegisteredNameTuple(Class<?> cls) {
+    String name = extRegistry.registeredClasses.inverse().get(cls);
+    int index = name.lastIndexOf(".");
+    if (index != -1) {
+      return Tuple2.of(name.substring(0, index), name.substring(index + 1));
+    } else {
+      return Tuple2.of("", name);
+    }
+  }
+
+  public boolean isRegisteredById(Class<?> cls) {
     return extRegistry.registeredClassIdMap.get(cls) != null;
   }
 
@@ -525,6 +587,10 @@ public class ClassResolver {
     return null;
   }
 
+  public Class<?> getRegisteredClass(String className) {
+    return extRegistry.registeredClasses.get(className);
+  }
+
   public List<Class<?>> getRegisteredClasses() {
     return Arrays.stream(registeredId2ClassInfo)
         .filter(Objects::nonNull)
@@ -532,12 +598,42 @@ public class ClassResolver {
         .collect(Collectors.toList());
   }
 
+  public String getTypeAlias(Class<?> cls) {
+    Short id = extRegistry.registeredClassIdMap.get(cls);
+    if (id != null) {
+      return String.valueOf(id);
+    }
+    String name = extRegistry.registeredClasses.inverse().get(cls);
+    if (name != null) {
+      return name;
+    }
+    return cls.getName();
+  }
+
   /**
    * Mark non-inner registered final types as non-final to write class def for those types. Note if
    * a class is registered but not an inner class with inner serializer, it will still be taken as
    * non-final to write class def, so that it can be deserialized by the peer still.
    */
   public boolean isMonomorphic(Class<?> clz) {
+    if (fury.isCrossLanguage()) {
+      if (TypeUtils.unwrap(clz).isPrimitive() || clz.isEnum() || clz == String.class) {
+        return true;
+      }
+      if (clz.isArray() && TypeUtils.getArrayComponent(clz).isPrimitive()) {
+        return true;
+      }
+      ClassInfo classInfo = xtypeResolver.getClassInfo(clz, false);
+      if (classInfo != null) {
+        if (classInfo.serializer instanceof TimeSerializers.TimeSerializer) {
+          return true;
+        }
+        if (classInfo.serializer instanceof TimeSerializers.ImmutableTimeSerializer) {
+          return true;
+        }
+      }
+      return false;
+    }
     if (fury.getConfig().isMetaShareEnabled()) {
       // can't create final map/collection type using TypeUtils.mapOf(TypeToken<K>,
       // TypeToken<V>)
@@ -734,18 +830,6 @@ public class ClassResolver {
   /** Ass serializer for specified class. */
   private void addSerializer(Class<?> type, Serializer<?> serializer) {
     Preconditions.checkNotNull(serializer);
-    String typeTag = null;
-    short typeId = serializer.getXtypeId();
-    if (typeId != Fury.NOT_SUPPORT_CROSS_LANGUAGE) {
-      if (typeId > Fury.NOT_SUPPORT_CROSS_LANGUAGE) {
-        typeIdToClassXLangMap.put(typeId, type);
-      }
-      if (typeId == Fury.FURY_TYPE_TAG_ID) {
-        typeTag = serializer.getCrossLanguageTypeTag();
-        typeTagToClassXLangMap.put(typeTag, type);
-      }
-    }
-
     // 1. Try to get ClassInfo from `registeredId2ClassInfo` and
     // `classInfoMap` or create a new `ClassInfo`.
     ClassInfo classInfo;
@@ -763,8 +847,8 @@ public class ClassResolver {
       classInfo = classInfoMap.get(type);
     }
 
-    if (classInfo == null || typeTag != null || classId != classInfo.classId) {
-      classInfo = new ClassInfo(this, type, typeTag, null, classId);
+    if (classInfo == null || classId != classInfo.classId) {
+      classInfo = new ClassInfo(this, type, null, classId, (short) 0);
       classInfoMap.put(type, classInfo);
       if (registered) {
         registeredId2ClassInfo[classId] = classInfo;
@@ -973,7 +1057,25 @@ public class ClassResolver {
     }
   }
 
+  public boolean isSet(Class<?> cls) {
+    if (Set.class.isAssignableFrom(cls)) {
+      return true;
+    }
+    if (fury.getConfig().isScalaOptimizationEnabled()) {
+      // Scala map is scala iterable too.
+      if (ScalaTypes.getScalaMapType().isAssignableFrom(cls)) {
+        return false;
+      }
+      return ScalaTypes.getScalaSetType().isAssignableFrom(cls);
+    } else {
+      return false;
+    }
+  }
+
   public boolean isMap(Class<?> cls) {
+    if (cls == NonexistentMetaShared.class) {
+      return false;
+    }
     return Map.class.isAssignableFrom(cls)
         || (fury.getConfig().isScalaOptimizationEnabled()
             && ScalaTypes.getScalaMapType().isAssignableFrom(cls));
@@ -1119,6 +1221,7 @@ public class ClassResolver {
   }
 
   // Invoked by fury JIT.
+  @Override
   public ClassInfo getClassInfo(Class<?> cls) {
     ClassInfo classInfo = classInfoMap.get(cls);
     if (classInfo == null || classInfo.serializer == null) {
@@ -1150,6 +1253,7 @@ public class ClassResolver {
    * @param createClassInfoIfNotFound whether create class info if not found.
    * @return Class info.
    */
+  @Override
   public ClassInfo getClassInfo(Class<?> cls, boolean createClassInfoIfNotFound) {
     if (createClassInfoIfNotFound) {
       return getOrUpdateClassInfo(cls);
@@ -1161,6 +1265,10 @@ public class ClassResolver {
     }
   }
 
+  void setClassInfo(Class<?> cls, ClassInfo classInfo) {
+    classInfoMap.put(cls, classInfo);
+  }
+
   @Internal
   public ClassInfo getOrUpdateClassInfo(Class<?> cls) {
     ClassInfo classInfo = classInfoCache;
@@ -1273,7 +1381,7 @@ public class ClassResolver {
   }
 
   private boolean isSecure(Class<?> cls) {
-    if (extRegistry.registeredClassIdMap.containsKey(cls) || shimDispatcher.contains(cls)) {
+    if (extRegistry.registeredClasses.inverse().containsKey(cls) || shimDispatcher.contains(cls)) {
       return true;
     }
     if (cls.isArray()) {
@@ -1297,7 +1405,7 @@ public class ClassResolver {
 
   /**
    * Write class info to <code>buffer</code>. TODO(chaokunyang): The method should try to write
-   * aligned data to reduce cpu instruction overhead. `writeClass` is the last step before
+   * aligned data to reduce cpu instruction overhead. `writeClassInfo` is the last step before
    * serializing object, if this writes are aligned, then later serialization will be more
    * efficient.
    */
@@ -1308,34 +1416,35 @@ public class ClassResolver {
     } else if (cls == Long.class) {
       buffer.writeVarUint32Small7(LONG_CLASS_ID << 1);
     } else {
-      writeClass(buffer, getOrUpdateClassInfo(cls));
+      writeClassInfo(buffer, getOrUpdateClassInfo(cls));
     }
   }
 
   // The jit-compiled native code fot this method will be too big for inline, so we generated
   // `getClassInfo`
   // in fury-jit, see `BaseSeqCodecBuilder#writeAndGetClassInfo`
-  // public ClassInfo writeClass(MemoryBuffer buffer, Class<?> cls, ClassInfoHolder classInfoHolder)
+  // public ClassInfo writeClassInfo(MemoryBuffer buffer, Class<?> cls, ClassInfoHolder
+  // classInfoHolder)
   // {
   //   ClassInfo classInfo = getClassInfo(cls, classInfoHolder);
-  //   writeClass(buffer, classInfo);
+  //   writeClassInfo(buffer, classInfo);
   //   return classInfo;
   // }
 
   /** Write classname for java serialization. */
-  public void writeClass(MemoryBuffer buffer, ClassInfo classInfo) {
+  public void writeClassInfo(MemoryBuffer buffer, ClassInfo classInfo) {
     if (metaContextShareEnabled) {
       // FIXME(chaokunyang) Register class but not register serializer can't be used with
       //  meta share mode, because no class def are sent to peer.
-      writeClassWithMetaShare(buffer, classInfo);
+      writeClassInfoWithMetaShare(buffer, classInfo);
     } else {
       if (classInfo.classId == NO_CLASS_ID) { // no class id provided.
         // use classname
         // if it's null, it's a bug.
-        assert classInfo.packageNameBytes != null;
-        metaStringResolver.writeMetaStringBytesWithFlag(buffer, classInfo.packageNameBytes);
-        assert classInfo.classNameBytes != null;
-        metaStringResolver.writeMetaStringBytes(buffer, classInfo.classNameBytes);
+        assert classInfo.namespaceBytes != null;
+        metaStringResolver.writeMetaStringBytesWithFlag(buffer, classInfo.namespaceBytes);
+        assert classInfo.typeNameBytes != null;
+        metaStringResolver.writeMetaStringBytes(buffer, classInfo.typeNameBytes);
       } else {
         // use classId
         buffer.writeVarUint32(classInfo.classId << 1);
@@ -1343,7 +1452,7 @@ public class ClassResolver {
     }
   }
 
-  public void writeClassWithMetaShare(MemoryBuffer buffer, ClassInfo classInfo) {
+  public void writeClassInfoWithMetaShare(MemoryBuffer buffer, ClassInfo classInfo) {
     if (classInfo.classId != NO_CLASS_ID && !classInfo.needToWriteClassDef) {
       buffer.writeVarUint32(classInfo.classId << 1);
       return;
@@ -1467,7 +1576,7 @@ public class ClassResolver {
     Class<?> cls = clz;
     Short classId = extRegistry.registeredClassIdMap.get(cls);
     ClassInfo classInfo =
-        new ClassInfo(this, cls, null, null, classId == null ? NO_CLASS_ID : classId);
+        new ClassInfo(this, cls, null, classId == null ? NO_CLASS_ID : classId, NOT_SUPPORT_XLANG);
     classInfo.classDef = classDef;
     if (NonexistentClass.class.isAssignableFrom(TypeUtils.getComponentIfArray(cls))) {
       if (cls == NonexistentMetaShared.class) {
@@ -1598,7 +1707,7 @@ public class ClassResolver {
   }
 
   /**
-   * Native code for ClassResolver.writeClass is too big to inline, so inline it manually.
+   * Native code for ClassResolver.writeClassInfo is too big to inline, so inline it manually.
    *
    * <p>See `already compiled into a big method` in <a
    * href="https://wiki.openjdk.org/display/HotSpot/Server+Compiler+Inlining+Messages">Server+Compiler+Inlining+Messages</a>
@@ -1606,7 +1715,7 @@ public class ClassResolver {
   // Note: Thread safe fot jit thread to call.
   public Expression writeClassExpr(
       Expression classResolverRef, Expression buffer, Expression classInfo) {
-    return new Invoke(classResolverRef, "writeClass", buffer, classInfo);
+    return new Invoke(classResolverRef, "writeClassInfo", buffer, classInfo);
   }
 
   // Note: Thread safe fot jit thread to call.
@@ -1628,7 +1737,7 @@ public class ClassResolver {
   /**
    * Write classname for java serialization. Note that the object of provided class can be
    * non-serializable, and class with writeReplace/readResolve defined won't be skipped. For
-   * serializable object, {@link #writeClass(MemoryBuffer, ClassInfo)} should be invoked.
+   * serializable object, {@link #writeClassInfo(MemoryBuffer, ClassInfo)} should be invoked.
    */
   public void writeClassInternal(MemoryBuffer buffer, Class<?> cls) {
     ClassInfo classInfo = classInfoMap.get(cls);
@@ -1636,7 +1745,9 @@ public class ClassResolver {
       Short classId = extRegistry.registeredClassIdMap.get(cls);
       // Don't create serializer in case the object for class is non-serializable,
       // Or class is abstract or interface.
-      classInfo = new ClassInfo(this, cls, null, null, classId == null ? NO_CLASS_ID : classId);
+      classInfo =
+          new ClassInfo(
+              this, cls, null, classId == null ? NO_CLASS_ID : classId, NOT_SUPPORT_XLANG);
       classInfoMap.put(cls, classInfo);
     }
     writeClassInternal(buffer, classInfo);
@@ -1654,8 +1765,8 @@ public class ClassResolver {
     } else {
       // let the lowermost bit of next byte be set, so the deserialization can know
       // whether need to read class by name in advance
-      metaStringResolver.writeMetaStringBytesWithFlag(buffer, classInfo.packageNameBytes);
-      metaStringResolver.writeMetaStringBytes(buffer, classInfo.classNameBytes);
+      metaStringResolver.writeMetaStringBytesWithFlag(buffer, classInfo.namespaceBytes);
+      metaStringResolver.writeMetaStringBytes(buffer, classInfo.typeNameBytes);
     }
     classInfo.classId = classId;
   }
@@ -1707,6 +1818,7 @@ public class ClassResolver {
    * reduce map lookup to load class from binary.
    */
   @CodegenInvoke
+  @Override
   public ClassInfo readClassInfo(MemoryBuffer buffer, ClassInfo classInfoCache) {
     if (metaContextShareEnabled) {
       return readClassInfoWithMetaShare(buffer, fury.getSerializationContext().getMetaContext());
@@ -1753,44 +1865,43 @@ public class ClassResolver {
 
   private ClassInfo readClassInfoFromBytes(
       MemoryBuffer buffer, ClassInfo classInfoCache, int header) {
-    MetaStringBytes simpleClassNameBytesCache = classInfoCache.classNameBytes;
-    MetaStringBytes packageBytes;
+    MetaStringBytes typeNameBytesCache = classInfoCache.typeNameBytes;
+    MetaStringBytes namespaceBytes;
     MetaStringBytes simpleClassNameBytes;
-    if (simpleClassNameBytesCache != null) {
-      MetaStringBytes packageNameBytesCache = classInfoCache.packageNameBytes;
-      packageBytes =
+    if (typeNameBytesCache != null) {
+      MetaStringBytes packageNameBytesCache = classInfoCache.namespaceBytes;
+      namespaceBytes =
           metaStringResolver.readMetaStringBytesWithFlag(buffer, packageNameBytesCache, header);
       assert packageNameBytesCache != null;
-      simpleClassNameBytes =
-          metaStringResolver.readMetaStringBytes(buffer, simpleClassNameBytesCache);
-      if (simpleClassNameBytesCache.hashCode == simpleClassNameBytes.hashCode
-          && packageNameBytesCache.hashCode == packageBytes.hashCode) {
+      simpleClassNameBytes = metaStringResolver.readMetaStringBytes(buffer, typeNameBytesCache);
+      if (typeNameBytesCache.hashCode == simpleClassNameBytes.hashCode
+          && packageNameBytesCache.hashCode == namespaceBytes.hashCode) {
         return classInfoCache;
       }
     } else {
-      packageBytes = metaStringResolver.readMetaStringBytesWithFlag(buffer, header);
+      namespaceBytes = metaStringResolver.readMetaStringBytesWithFlag(buffer, header);
       simpleClassNameBytes = metaStringResolver.readMetaStringBytes(buffer);
     }
-    ClassInfo classInfo = loadBytesToClassInfo(packageBytes, simpleClassNameBytes);
+    ClassInfo classInfo = loadBytesToClassInfo(namespaceBytes, simpleClassNameBytes);
     if (classInfo.serializer == null) {
       return getClassInfo(classInfo.cls);
     }
     return classInfo;
   }
 
-  private ClassInfo loadBytesToClassInfo(
+  ClassInfo loadBytesToClassInfo(
       MetaStringBytes packageBytes, MetaStringBytes simpleClassNameBytes) {
-    ClassNameBytes classNameBytes =
-        new ClassNameBytes(packageBytes.hashCode, simpleClassNameBytes.hashCode);
-    ClassInfo classInfo = compositeClassNameBytes2ClassInfo.get(classNameBytes);
+    TypeNameBytes typeNameBytes =
+        new TypeNameBytes(packageBytes.hashCode, simpleClassNameBytes.hashCode);
+    ClassInfo classInfo = compositeNameBytes2ClassInfo.get(typeNameBytes);
     if (classInfo == null) {
-      classInfo = populateBytesToClassInfo(classNameBytes, packageBytes, simpleClassNameBytes);
+      classInfo = populateBytesToClassInfo(typeNameBytes, packageBytes, simpleClassNameBytes);
     }
     return classInfo;
   }
 
   private ClassInfo populateBytesToClassInfo(
-      ClassNameBytes classNameBytes,
+      TypeNameBytes typeNameBytes,
       MetaStringBytes packageBytes,
       MetaStringBytes simpleClassNameBytes) {
     String packageName = packageBytes.decode(PACKAGE_DECODER);
@@ -1808,8 +1919,8 @@ public class ClassResolver {
             simpleClassNameBytes,
             false,
             null,
-            null,
-            NO_CLASS_ID);
+            NO_CLASS_ID,
+            NOT_SUPPORT_XLANG);
     if (NonexistentClass.class.isAssignableFrom(TypeUtils.getComponentIfArray(cls))) {
       classInfo.serializer =
           NonexistentClassSerializers.getSerializer(fury, classSpec.entireClassName, cls);
@@ -1820,43 +1931,14 @@ public class ClassResolver {
         classInfoMap.put(cls, classInfo);
       }
     }
-    compositeClassNameBytes2ClassInfo.put(classNameBytes, classInfo);
+    compositeNameBytes2ClassInfo.put(typeNameBytes, classInfo);
     return classInfo;
   }
 
-  public void xwriteClass(MemoryBuffer buffer, Class<?> cls) {
-    metaStringResolver.writeMetaStringBytes(buffer, getOrUpdateClassInfo(cls).fullClassNameBytes);
-  }
-
-  public void xwriteTypeTag(MemoryBuffer buffer, Class<?> cls) {
-    metaStringResolver.writeMetaStringBytes(buffer, getOrUpdateClassInfo(cls).typeTagBytes);
-  }
-
-  public Class<?> xreadClass(MemoryBuffer buffer) {
-    MetaStringBytes byteString = metaStringResolver.readMetaStringBytes(buffer);
-    Class<?> cls = classNameBytes2Class.get(byteString);
-    if (cls == null) {
-      Preconditions.checkNotNull(byteString);
-      String className = byteString.decode(Encoders.GENERIC_DECODER);
-      cls = loadClass(className);
-      classNameBytes2Class.put(byteString, cls);
-    }
-    currentReadClass = cls;
-    return cls;
-  }
-
-  public String xreadClassName(MemoryBuffer buffer) {
-    return metaStringResolver.readMetaString(buffer);
-  }
-
   public Class<?> getCurrentReadClass() {
     return currentReadClass;
   }
 
-  private Class<?> loadClass(String className) {
-    return loadClass(className, false, 0);
-  }
-
   private Class<?> loadClass(ClassSpec classSpec) {
     return loadClass(classSpec.entireClassName, classSpec.isEnum, classSpec.dimension);
   }
@@ -1868,6 +1950,10 @@ public class ClassResolver {
   private Class<?> loadClass(
       String className, boolean isEnum, int arrayDims, boolean deserializeNonexistentClass) {
     extRegistry.classChecker.checkClass(this, className);
+    Class<?> cls = extRegistry.registeredClasses.get(className);
+    if (cls != null) {
+      return cls;
+    }
     try {
       return Class.forName(className, false, fury.getClassLoader());
     } catch (ClassNotFoundException e) {
@@ -1897,39 +1983,6 @@ public class ClassResolver {
 
   public void resetWrite() {}
 
-  public Class<?> getClassByTypeId(short typeId) {
-    return typeIdToClassXLangMap.get(typeId);
-  }
-
-  public Class<?> readClassByTypeTag(MemoryBuffer buffer) {
-    String tag = metaStringResolver.readMetaString(buffer);
-    return typeTagToClassXLangMap.get(tag);
-  }
-
-  private static class ClassNameBytes {
-    private final long packageHash;
-    private final long classNameHash;
-
-    private ClassNameBytes(long packageHash, long classNameHash) {
-      this.packageHash = packageHash;
-      this.classNameHash = classNameHash;
-    }
-
-    @Override
-    public boolean equals(Object o) {
-      // ClassNameBytes is used internally, skip
-      ClassNameBytes that = (ClassNameBytes) o;
-      return packageHash == that.packageHash && classNameHash == that.classNameHash;
-    }
-
-    @Override
-    public int hashCode() {
-      int result = 31 + (int) (packageHash ^ (packageHash >>> 32));
-      result = result * 31 + (int) (classNameHash ^ (classNameHash >>> 32));
-      return result;
-    }
-  }
-
   public GenericType buildGenericType(TypeRef<?> typeRef) {
     return GenericType.build(
         typeRef,
@@ -1943,15 +1996,26 @@ public class ClassResolver {
   }
 
   public GenericType buildGenericType(Type type) {
-    return GenericType.build(
-        type,
-        t -> {
-          if (t.getClass() == Class.class) {
-            return isMonomorphic((Class<?>) t);
-          } else {
-            return isMonomorphic(getRawType(t));
-          }
-        });
+    GenericType genericType = extRegistry.genericTypes.get(type);
+    if (genericType != null) {
+      return genericType;
+    }
+    return populateGenericType(type);
+  }
+
+  private GenericType populateGenericType(Type type) {
+    GenericType genericType =
+        GenericType.build(
+            type,
+            t -> {
+              if (t.getClass() == Class.class) {
+                return isMonomorphic((Class<?>) t);
+              } else {
+                return isMonomorphic(getRawType(t));
+              }
+            });
+    extRegistry.genericTypes.put(type, genericType);
+    return genericType;
   }
 
   public GenericType getObjectGenericType() {
@@ -1959,12 +2023,12 @@ public class ClassResolver {
   }
 
   public ClassInfo newClassInfo(Class<?> cls, Serializer<?> serializer, short classId) {
-    return new ClassInfo(this, cls, null, serializer, classId);
+    return new ClassInfo(this, cls, serializer, classId, NOT_SUPPORT_XLANG);
   }
 
   // Invoked by fury JIT.
   public ClassInfo nilClassInfo() {
-    return new ClassInfo(this, null, null, null, NO_CLASS_ID);
+    return new ClassInfo(this, null, null, NO_CLASS_ID, NOT_SUPPORT_XLANG);
   }
 
   public ClassInfoHolder nilClassInfoHolder() {
@@ -1993,6 +2057,65 @@ public class ClassResolver {
     extRegistry.codeGeneratorMap.put(Arrays.asList(loaders), codeGenerator);
   }
 
+  public DescriptorGrouper createDescriptorGrouper(
+      Collection<Descriptor> descriptors, boolean descriptorsGroupedOrdered) {
+    return createDescriptorGrouper(descriptors, descriptorsGroupedOrdered, null);
+  }
+
+  public DescriptorGrouper createDescriptorGrouper(
+      Collection<Descriptor> descriptors,
+      boolean descriptorsGroupedOrdered,
+      Function<Descriptor, Descriptor> descriptorUpdator) {
+    if (fury.isCrossLanguage()) {
+      return DescriptorGrouper.createDescriptorGrouper(
+          this::isMonomorphic,
+          descriptors,
+          descriptorsGroupedOrdered,
+          descriptorUpdator,
+          fury.compressInt(),
+          fury.compressLong(),
+          (o1, o2) -> {
+            int xtypeId = getXtypeId(o1.getRawType());
+            int xtypeId2 = getXtypeId(o2.getRawType());
+            if (xtypeId == xtypeId2) {
+              return o1.getSnakeCaseName().compareTo(o2.getSnakeCaseName());
+            } else {
+              return xtypeId - xtypeId2;
+            }
+          });
+    }
+    return DescriptorGrouper.createDescriptorGrouper(
+        fury.getClassResolver()::isMonomorphic,
+        descriptors,
+        descriptorsGroupedOrdered,
+        descriptorUpdator,
+        fury.compressInt(),
+        fury.compressLong(),
+        DescriptorGrouper.COMPARATOR_BY_TYPE_AND_NAME);
+  }
+
+  private static final int UNKNOWN_TYPE_ID = -1;
+
+  private int getXtypeId(Class<?> cls) {
+    if (isCollection(cls)) {
+      return Types.LIST;
+    }
+    if (cls.isArray() && !cls.getComponentType().isPrimitive()) {
+      return Types.LIST;
+    }
+    if (isMap(cls)) {
+      return Types.MAP;
+    }
+    if (fury.getXtypeResolver().isRegistered(cls)) {
+      return fury.getXtypeResolver().getClassInfo(cls).getXtypeId();
+    } else {
+      if (ReflectionUtils.isMonomorphic(cls)) {
+        throw new UnsupportedOperationException(cls + " is not supported for xlang serialization");
+      }
+      return UNKNOWN_TYPE_ID;
+    }
+  }
+
   public Fury getFury() {
     return fury;
   }
