@@ -50,8 +50,8 @@ import org.apache.fury.reflect.FieldAccessor;
 import org.apache.fury.reflect.ReflectionUtils;
 import org.apache.fury.reflect.TypeRef;
 import org.apache.fury.serializer.PrimitiveSerializers;
-import org.apache.fury.serializer.collection.CollectionSerializer;
-import org.apache.fury.serializer.collection.MapSerializer;
+import org.apache.fury.serializer.collection.AbstractCollectionSerializer;
+import org.apache.fury.serializer.collection.AbstractMapSerializer;
 import org.apache.fury.type.Descriptor;
 import org.apache.fury.type.TypeUtils;
 import org.apache.fury.util.MurmurHash3;
@@ -567,30 +567,56 @@ public class FieldResolver {
     if (fieldType == FieldTypes.COLLECTION_ELEMENT_FINAL) {
       ClassInfo elementClassInfo = classResolver.readClassInfo(buffer, classInfoHolder);
       ClassInfo classInfo = classResolver.readClassInfo(buffer, classInfoHolder);
-      CollectionSerializer collectionSerializer = (CollectionSerializer) classInfo.getSerializer();
-      collectionSerializer.setElementSerializer(elementClassInfo.getSerializer());
-      o = collectionSerializer.read(buffer);
+      AbstractCollectionSerializer collectionSerializer =
+          (AbstractCollectionSerializer) classInfo.getSerializer();
+      try {
+        collectionSerializer.setElementSerializer(elementClassInfo.getSerializer());
+        o = collectionSerializer.read(buffer);
+      } finally {
+        // Some collectionSerializer may overwrite write/read method, then clear element serializer
+        // may not got invoked.
+        collectionSerializer.setElementSerializer(null);
+      }
     } else if (fieldType == FieldTypes.MAP_KV_FINAL) {
       ClassInfo keyClassInfo = classResolver.readClassInfo(buffer, classInfoHolder);
       ClassInfo valueClassInfo = classResolver.readClassInfo(buffer, classInfoHolder);
       ClassInfo classInfo = classResolver.readClassInfo(buffer, classInfoHolder);
-      MapSerializer mapSerializer = (MapSerializer) classInfo.getSerializer();
-      mapSerializer.setKeySerializer(keyClassInfo.getSerializer());
-      mapSerializer.setValueSerializer(valueClassInfo.getSerializer());
-      o = mapSerializer.read(buffer);
+      AbstractMapSerializer mapSerializer = (AbstractMapSerializer) classInfo.getSerializer();
+      try {
+        mapSerializer.setKeySerializer(keyClassInfo.getSerializer());
+        mapSerializer.setValueSerializer(valueClassInfo.getSerializer());
+        o = mapSerializer.read(buffer);
+      } finally {
+        // Some mmapSerializer may overwrite write/read method, then clear serializer
+        // may not got invoked.
+        mapSerializer.setKeySerializer(null);
+        mapSerializer.setValueSerializer(null);
+      }
     } else if (fieldType == FieldTypes.MAP_KEY_FINAL) {
       ClassInfo keyClassInfo = classResolver.readClassInfo(buffer, classInfoHolder);
       ClassInfo classInfo = classResolver.readClassInfo(buffer, classInfoHolder);
-      MapSerializer mapSerializer = (MapSerializer) classInfo.getSerializer();
-      mapSerializer.setKeySerializer(keyClassInfo.getSerializer());
-      o = mapSerializer.read(buffer);
+      AbstractMapSerializer mapSerializer = (AbstractMapSerializer) classInfo.getSerializer();
+      try {
+        mapSerializer.setKeySerializer(keyClassInfo.getSerializer());
+        o = mapSerializer.read(buffer);
+      } finally {
+        // Some mmapSerializer may overwrite write/read method, then clear serializer
+        // may not got invoked.
+        mapSerializer.setKeySerializer(null);
+      }
     } else {
       Preconditions.checkArgument(fieldType == FieldTypes.MAP_VALUE_FINAL);
       ClassInfo valueClassInfo = classResolver.readClassInfo(buffer, classInfoHolder);
       ClassInfo classInfo = classResolver.readClassInfo(buffer, classInfoHolder);
-      MapSerializer mapSerializer = (MapSerializer) classInfo.getSerializer();
-      mapSerializer.setValueSerializer(valueClassInfo.getSerializer());
-      o = mapSerializer.read(buffer);
+      AbstractMapSerializer mapSerializer = (AbstractMapSerializer) classInfo.getSerializer();
+      try {
+        mapSerializer.setValueSerializer(valueClassInfo.getSerializer());
+        o = mapSerializer.read(buffer);
+      } finally {
+        // Some mmapSerializer may overwrite write/read method, then clear serializer
+        // may not got invoked.
+        mapSerializer.setValueSerializer(null);
+      }
     }
     return o;
   }
@@ -600,30 +626,56 @@ public class FieldResolver {
     if (fieldType == FieldTypes.COLLECTION_ELEMENT_FINAL) {
       ClassInfo elementClassInfo = classResolver.readClassInfo(buffer, classInfoHolder);
       ClassInfo classInfo = classResolver.readClassInfo(buffer, fieldInfo.getClassInfoHolder());
-      CollectionSerializer collectionSerializer = (CollectionSerializer) classInfo.getSerializer();
-      collectionSerializer.setElementSerializer(elementClassInfo.getSerializer());
-      o = collectionSerializer.read(buffer);
+      AbstractCollectionSerializer collectionSerializer =
+          (AbstractCollectionSerializer) classInfo.getSerializer();
+      try {
+        collectionSerializer.setElementSerializer(elementClassInfo.getSerializer());
+        o = collectionSerializer.read(buffer);
+        // Some collectionSerializer may overwrite write/read method, then clear element serializer
+        // may not got invoked.
+      } finally {
+        collectionSerializer.setElementSerializer(null);
+      }
     } else if (fieldType == FieldTypes.MAP_KV_FINAL) {
       ClassInfo keyClassInfo = classResolver.readClassInfo(buffer, classInfoHolder);
       ClassInfo valueClassInfo = classResolver.readClassInfo(buffer, classInfoHolder);
       ClassInfo classInfo = classResolver.readClassInfo(buffer, fieldInfo.getClassInfoHolder());
-      MapSerializer mapSerializer = (MapSerializer) classInfo.getSerializer();
-      mapSerializer.setKeySerializer(keyClassInfo.getSerializer());
-      mapSerializer.setValueSerializer(valueClassInfo.getSerializer());
-      o = mapSerializer.read(buffer);
+      AbstractMapSerializer mapSerializer = (AbstractMapSerializer) classInfo.getSerializer();
+      try {
+        mapSerializer.setKeySerializer(keyClassInfo.getSerializer());
+        mapSerializer.setValueSerializer(valueClassInfo.getSerializer());
+        o = mapSerializer.read(buffer);
+      } finally {
+        // Some mmapSerializer may overwrite write/read method, then clear serializer
+        // may not got invoked.
+        mapSerializer.setKeySerializer(null);
+        mapSerializer.setValueSerializer(null);
+      }
     } else if (fieldType == FieldTypes.MAP_KEY_FINAL) {
       ClassInfo keyClassInfo = classResolver.readClassInfo(buffer, classInfoHolder);
       ClassInfo classInfo = classResolver.readClassInfo(buffer, fieldInfo.getClassInfoHolder());
-      MapSerializer mapSerializer = (MapSerializer) classInfo.getSerializer();
-      mapSerializer.setKeySerializer(keyClassInfo.getSerializer());
-      o = mapSerializer.read(buffer);
+      AbstractMapSerializer mapSerializer = (AbstractMapSerializer) classInfo.getSerializer();
+      try {
+        mapSerializer.setKeySerializer(keyClassInfo.getSerializer());
+        o = mapSerializer.read(buffer);
+        // Some mmapSerializer may overwrite write/read method, then clear serializer
+        // may not got invoked.
+      } finally {
+        mapSerializer.setKeySerializer(null);
+      }
     } else {
       Preconditions.checkArgument(fieldType == FieldTypes.MAP_VALUE_FINAL);
       ClassInfo valueClassInfo = classResolver.readClassInfo(buffer, classInfoHolder);
       ClassInfo classInfo = classResolver.readClassInfo(buffer, fieldInfo.getClassInfoHolder());
-      MapSerializer mapSerializer = (MapSerializer) classInfo.getSerializer();
-      mapSerializer.setValueSerializer(valueClassInfo.getSerializer());
-      o = mapSerializer.read(buffer);
+      AbstractMapSerializer mapSerializer = (AbstractMapSerializer) classInfo.getSerializer();
+      try {
+        mapSerializer.setValueSerializer(valueClassInfo.getSerializer());
+        o = mapSerializer.read(buffer);
+        // Some mmapSerializer may overwrite write/read method, then clear serializer
+        // may not got invoked.
+      } finally {
+        mapSerializer.setValueSerializer(null);
+      }
     }
     return o;
   }
@@ -969,4 +1021,9 @@ public class FieldResolver {
       return valueType;
     }
   }
+
+  public static void main(String[] args) {
+    System.out.println(computeStringHash("list0") << 2);
+    System.out.println(computeStringHash("serializeListLast") << 2);
+  }
 }
