@@ -130,11 +130,12 @@ public final class CompatibleSerializer<T> extends CompatibleSerializerBase<T> {
       MemoryBuffer buffer, FieldResolver.FieldInfo fieldInfo, Object targetObject) {
     FieldAccessor fieldAccessor = fieldInfo.getFieldAccessor();
     short classId = fieldInfo.getEmbeddedClassId();
-    if (ObjectSerializer.writePrimitiveFieldValueFailed(
+    if (AbstractObjectSerializer.writePrimitiveFieldValueFailed(
         fury, buffer, targetObject, fieldAccessor, classId)) {
       Object fieldValue;
       fieldValue = fieldAccessor.getObject(targetObject);
-      if (ObjectSerializer.writeBasicObjectFieldValueFailed(fury, buffer, fieldValue, classId)) {
+      if (AbstractObjectSerializer.writeBasicObjectFieldValueFailed(
+          fury, buffer, fieldValue, classId)) {
         if (classId == ClassResolver.NO_CLASS_ID) { // SEPARATE_TYPES_HASH
           writeSeparateFieldValue(fieldInfo, buffer, fieldValue);
         } else {
@@ -221,10 +222,10 @@ public final class CompatibleSerializer<T> extends CompatibleSerializerBase<T> {
   private void writeCollectionField(
       MemoryBuffer buffer, FieldResolver.CollectionFieldInfo fieldInfo, Collection fieldValue) {
     ClassInfo elementClassInfo = fieldInfo.getElementClassInfo();
-    classResolver.writeClass(buffer, elementClassInfo);
+    classResolver.writeClassInfo(buffer, elementClassInfo);
     // following write is consistent with `BaseSeqCodecBuilder.serializeForCollection`
     ClassInfo classInfo = fieldInfo.getClassInfo(fieldValue.getClass());
-    classResolver.writeClass(buffer, classInfo);
+    classResolver.writeClassInfo(buffer, classInfo);
     AbstractCollectionSerializer collectionSerializer =
         (AbstractCollectionSerializer) classInfo.getSerializer();
     try {
@@ -241,11 +242,11 @@ public final class CompatibleSerializer<T> extends CompatibleSerializerBase<T> {
       MemoryBuffer buffer, FieldResolver.MapFieldInfo fieldInfo, Map fieldValue) {
     ClassInfo keyClassInfo = fieldInfo.getKeyClassInfo();
     ClassInfo valueClassInfo = fieldInfo.getValueClassInfo();
-    classResolver.writeClass(buffer, keyClassInfo);
-    classResolver.writeClass(buffer, valueClassInfo);
+    classResolver.writeClassInfo(buffer, keyClassInfo);
+    classResolver.writeClassInfo(buffer, valueClassInfo);
     // following write is consistent with `BaseSeqCodecBuilder.serializeForMap`
     ClassInfo classInfo = fieldInfo.getClassInfo(fieldValue.getClass());
-    classResolver.writeClass(buffer, classInfo);
+    classResolver.writeClassInfo(buffer, classInfo);
     AbstractMapSerializer mapSerializer = (AbstractMapSerializer) classInfo.getSerializer();
     try {
       mapSerializer.setKeySerializer(keyClassInfo.getSerializer());
@@ -262,10 +263,10 @@ public final class CompatibleSerializer<T> extends CompatibleSerializerBase<T> {
   private void writeMapKeyFinal(
       MemoryBuffer buffer, FieldResolver.MapFieldInfo fieldInfo, Map fieldValue) {
     ClassInfo keyClassInfo = fieldInfo.getKeyClassInfo();
-    classResolver.writeClass(buffer, keyClassInfo);
+    classResolver.writeClassInfo(buffer, keyClassInfo);
     // following write is consistent with `BaseSeqCodecBuilder.serializeForMap`
     ClassInfo classInfo = fieldInfo.getClassInfo(fieldValue.getClass());
-    classResolver.writeClass(buffer, classInfo);
+    classResolver.writeClassInfo(buffer, classInfo);
     AbstractMapSerializer mapSerializer = (AbstractMapSerializer) classInfo.getSerializer();
     try {
       mapSerializer.setKeySerializer(keyClassInfo.getSerializer());
@@ -280,10 +281,10 @@ public final class CompatibleSerializer<T> extends CompatibleSerializerBase<T> {
   private void writeMapValueFinal(
       MemoryBuffer buffer, FieldResolver.MapFieldInfo fieldInfo, Map fieldValue) {
     ClassInfo valueClassInfo = fieldInfo.getValueClassInfo();
-    classResolver.writeClass(buffer, valueClassInfo);
+    classResolver.writeClassInfo(buffer, valueClassInfo);
     // following write is consistent with `BaseSeqCodecBuilder.serializeForMap`
     ClassInfo classInfo = fieldInfo.getClassInfo(fieldValue.getClass());
-    classResolver.writeClass(buffer, classInfo);
+    classResolver.writeClassInfo(buffer, classInfo);
     AbstractMapSerializer mapSerializer = (AbstractMapSerializer) classInfo.getSerializer();
     try {
       mapSerializer.setValueSerializer(valueClassInfo.getSerializer());
@@ -558,9 +559,9 @@ public final class CompatibleSerializer<T> extends CompatibleSerializerBase<T> {
       FieldResolver.FieldInfo fieldInfo, MemoryBuffer buffer, Object targetObject) {
     FieldAccessor fieldAccessor = fieldInfo.getFieldAccessor();
     short classId = fieldInfo.getEmbeddedClassId();
-    if (ObjectSerializer.readPrimitiveFieldValueFailed(
+    if (AbstractObjectSerializer.readPrimitiveFieldValueFailed(
             fury, buffer, targetObject, fieldAccessor, classId)
-        && ObjectSerializer.readBasicObjectFieldValueFailed(
+        && AbstractObjectSerializer.readBasicObjectFieldValueFailed(
             fury, buffer, targetObject, fieldAccessor, classId)) {
       if (classId == ClassResolver.NO_CLASS_ID) {
         // SEPARATE_TYPES_HASH
