@@ -413,6 +413,7 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
    */
   private static class FuryObjectOutputStream extends ObjectOutputStream {
     private final Fury fury;
+    private final boolean compressInt;
     private final SlotsInfo slotsInfo;
     private MemoryBuffer buffer;
     private Object targetObject;
@@ -422,6 +423,7 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
       super();
       this.slotsInfo = slotsInfo;
       this.fury = slotsInfo.slotsSerializer.fury;
+      this.compressInt = fury.compressInt();
     }
 
     @Override
@@ -628,12 +630,16 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
 
     @Override
     public void writeInt(int v) throws IOException {
-      buffer.writeInt32(v);
+      if (compressInt) {
+        buffer.writeVarInt32(v);
+      } else {
+        buffer.writeInt32(v);
+      }
     }
 
     @Override
     public void writeLong(long v) throws IOException {
-      buffer.writeInt64(v);
+      fury.writeInt64(buffer, v);
     }
 
     @Override
@@ -692,6 +698,7 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
    */
   private static class FuryObjectInputStream extends ObjectInputStream {
     private final Fury fury;
+    private final boolean compressInt;
     private final SlotsInfo slotsInfo;
     private MemoryBuffer buffer;
     private Object targetObject;
@@ -701,6 +708,7 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
 
     protected FuryObjectInputStream(SlotsInfo slotsInfo) throws IOException {
       this.fury = slotsInfo.slotsSerializer.fury;
+      this.compressInt = fury.compressInt();
       this.slotsInfo = slotsInfo;
     }
 
@@ -933,12 +941,12 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
 
     @Override
     public int readInt() throws IOException {
-      return buffer.readInt32();
+      return compressInt ? buffer.readVarInt32() : buffer.readInt32();
     }
 
     @Override
     public long readLong() throws IOException {
-      return buffer.readInt64();
+      return fury.readInt64(buffer);
     }
 
     @Override
