@@ -48,7 +48,6 @@ import org.apache.fury.memory.MemoryBuffer;
 import org.apache.fury.memory.Platform;
 import org.apache.fury.reflect.ReflectionUtils;
 import org.apache.fury.resolver.ClassResolver;
-import org.apache.fury.type.Type;
 import org.apache.fury.util.ExceptionUtils;
 import org.apache.fury.util.GraalvmSupport;
 import org.apache.fury.util.StringUtils;
@@ -178,28 +177,18 @@ public class Serializers {
   }
 
   public abstract static class CrossLanguageCompatibleSerializer<T> extends Serializer<T> {
-    private final short typeId;
 
-    public CrossLanguageCompatibleSerializer(Fury fury, Class<T> cls, short typeId) {
+    public CrossLanguageCompatibleSerializer(Fury fury, Class<T> cls) {
       super(fury, cls);
-      this.typeId = typeId;
     }
 
     public CrossLanguageCompatibleSerializer(
-        Fury fury, Class<T> cls, short typeId, boolean needToWriteRef) {
-      super(fury, cls, needToWriteRef);
-      this.typeId = typeId;
-    }
-
-    public CrossLanguageCompatibleSerializer(
-        Fury fury, Class<T> cls, short typeId, boolean needToWriteRef, boolean immutable) {
+        Fury fury, Class<T> cls, boolean needToWriteRef, boolean immutable) {
       super(fury, cls, needToWriteRef, immutable);
-      this.typeId = typeId;
     }
 
-    @Override
-    public short getXtypeId() {
-      return typeId;
+    public CrossLanguageCompatibleSerializer(Fury fury, Class<T> cls, boolean needToWriteRef) {
+      super(fury, cls, needToWriteRef);
     }
 
     @Override
@@ -239,12 +228,7 @@ public class Serializers {
 
     @Override
     public void xwrite(MemoryBuffer buffer, T value) {
-      stringSerializer.writeUTF8String(buffer, value.toString());
-    }
-
-    @Override
-    public short getXtypeId() {
-      return (short) -Type.STRING.getId();
+      stringSerializer.writeString(buffer, value.toString());
     }
 
     @Override
@@ -265,7 +249,7 @@ public class Serializers {
       } else {
         char[] v = (char[]) GET_VALUE.apply(value);
         if (StringUtils.isLatin(v)) {
-          stringSerializer.writeCharsLatin(buffer, v, value.length());
+          stringSerializer.writeCharsLatin1(buffer, v, value.length());
         } else {
           stringSerializer.writeCharsUTF16(buffer, v, value.length());
         }
@@ -292,7 +276,7 @@ public class Serializers {
 
     @Override
     public StringBuilder xread(MemoryBuffer buffer) {
-      return new StringBuilder(stringSerializer.readUTF8String(buffer));
+      return new StringBuilder(stringSerializer.readString(buffer));
     }
   }
 
@@ -315,7 +299,7 @@ public class Serializers {
 
     @Override
     public StringBuffer xread(MemoryBuffer buffer) {
-      return new StringBuffer(stringSerializer.readUTF8String(buffer));
+      return new StringBuffer(stringSerializer.readString(buffer));
     }
   }
 
@@ -577,19 +561,20 @@ public class Serializers {
   }
 
   public static void registerDefaultSerializers(Fury fury) {
-    fury.registerSerializer(Class.class, new ClassSerializer(fury));
-    fury.registerSerializer(StringBuilder.class, new StringBuilderSerializer(fury));
-    fury.registerSerializer(StringBuffer.class, new StringBufferSerializer(fury));
-    fury.registerSerializer(BigInteger.class, new BigIntegerSerializer(fury));
-    fury.registerSerializer(BigDecimal.class, new BigDecimalSerializer(fury));
-    fury.registerSerializer(AtomicBoolean.class, new AtomicBooleanSerializer(fury));
-    fury.registerSerializer(AtomicInteger.class, new AtomicIntegerSerializer(fury));
-    fury.registerSerializer(AtomicLong.class, new AtomicLongSerializer(fury));
-    fury.registerSerializer(AtomicReference.class, new AtomicReferenceSerializer(fury));
-    fury.registerSerializer(Currency.class, new CurrencySerializer(fury));
-    fury.registerSerializer(URI.class, new URISerializer(fury));
-    fury.registerSerializer(Pattern.class, new RegexSerializer(fury));
-    fury.registerSerializer(UUID.class, new UUIDSerializer(fury));
-    fury.registerSerializer(Object.class, new EmptyObjectSerializer(fury));
+    ClassResolver resolver = fury.getClassResolver();
+    resolver.registerSerializer(Class.class, new ClassSerializer(fury));
+    resolver.registerSerializer(StringBuilder.class, new StringBuilderSerializer(fury));
+    resolver.registerSerializer(StringBuffer.class, new StringBufferSerializer(fury));
+    resolver.registerSerializer(BigInteger.class, new BigIntegerSerializer(fury));
+    resolver.registerSerializer(BigDecimal.class, new BigDecimalSerializer(fury));
+    resolver.registerSerializer(AtomicBoolean.class, new AtomicBooleanSerializer(fury));
+    resolver.registerSerializer(AtomicInteger.class, new AtomicIntegerSerializer(fury));
+    resolver.registerSerializer(AtomicLong.class, new AtomicLongSerializer(fury));
+    resolver.registerSerializer(AtomicReference.class, new AtomicReferenceSerializer(fury));
+    resolver.registerSerializer(Currency.class, new CurrencySerializer(fury));
+    resolver.registerSerializer(URI.class, new URISerializer(fury));
+    resolver.registerSerializer(Pattern.class, new RegexSerializer(fury));
+    resolver.registerSerializer(UUID.class, new UUIDSerializer(fury));
+    resolver.registerSerializer(Object.class, new EmptyObjectSerializer(fury));
   }
 }
