@@ -41,18 +41,29 @@ import java.util.GregorianCalendar;
 import java.util.TimeZone;
 import org.apache.fury.Fury;
 import org.apache.fury.memory.MemoryBuffer;
-import org.apache.fury.type.Type;
+import org.apache.fury.resolver.ClassResolver;
 import org.apache.fury.util.DateTimeUtils;
 
 /** Serializers for all time related types. */
 public class TimeSerializers {
-  public abstract static class TimeSerializer<T> extends ImmutableSerializer<T> {
+  public abstract static class TimeSerializer<T> extends Serializer<T> {
 
     public TimeSerializer(Fury fury, Class<T> type) {
-      super(fury, type, !fury.getConfig().isTimeRefIgnored());
+      super(fury, type, !fury.getConfig().isTimeRefIgnored(), false);
     }
 
     public TimeSerializer(Fury fury, Class<T> type, boolean needToWriteRef) {
+      super(fury, type, needToWriteRef, false);
+    }
+  }
+
+  public abstract static class ImmutableTimeSerializer<T> extends ImmutableSerializer<T> {
+
+    public ImmutableTimeSerializer(Fury fury, Class<T> type) {
+      super(fury, type, !fury.getConfig().isTimeRefIgnored());
+    }
+
+    public ImmutableTimeSerializer(Fury fury, Class<T> type, boolean needToWriteRef) {
       super(fury, type, needToWriteRef);
     }
   }
@@ -76,6 +87,16 @@ public class TimeSerializers {
       return newInstance(buffer.readInt64());
     }
 
+    @Override
+    public void xwrite(MemoryBuffer buffer, T value) {
+      buffer.writeInt64(value.getTime());
+    }
+
+    @Override
+    public T xread(MemoryBuffer buffer) {
+      return newInstance(buffer.readInt64());
+    }
+
     protected abstract T newInstance(long time);
   }
 
@@ -92,6 +113,11 @@ public class TimeSerializers {
     protected Date newInstance(long time) {
       return new Date(time);
     }
+
+    @Override
+    public Date copy(Date value) {
+      return newInstance(value.getTime());
+    }
   }
 
   public static final class SqlDateSerializer extends BaseDateSerializer<java.sql.Date> {
@@ -107,6 +133,11 @@ public class TimeSerializers {
     protected java.sql.Date newInstance(long time) {
       return new java.sql.Date(time);
     }
+
+    @Override
+    public java.sql.Date copy(java.sql.Date value) {
+      return newInstance(value.getTime());
+    }
   }
 
   public static final class SqlTimeSerializer extends BaseDateSerializer<Time> {
@@ -123,20 +154,22 @@ public class TimeSerializers {
     protected Time newInstance(long time) {
       return new Time(time);
     }
+
+    @Override
+    public Time copy(Time value) {
+      return newInstance(value.getTime());
+    }
   }
 
   public static final class TimestampSerializer extends TimeSerializer<Timestamp> {
-    private final short typeId;
 
     public TimestampSerializer(Fury fury) {
       // conflict with instant
       super(fury, Timestamp.class);
-      typeId = (short) -Type.TIMESTAMP.getId();
     }
 
     public TimestampSerializer(Fury fury, boolean needToWriteRef) {
       super(fury, Timestamp.class, needToWriteRef);
-      typeId = (short) -Type.TIMESTAMP.getId();
     }
 
     @Override
@@ -149,11 +182,6 @@ public class TimeSerializers {
       return DateTimeUtils.toJavaTimestamp(buffer.readInt64());
     }
 
-    @Override
-    public short getXtypeId() {
-      return typeId;
-    }
-
     @Override
     public void write(MemoryBuffer buffer, Timestamp value) {
       long time = value.getTime() - (value.getNanos() / 1_000_000);
@@ -167,9 +195,14 @@ public class TimeSerializers {
       t.setNanos(buffer.readInt32());
       return t;
     }
+
+    @Override
+    public Timestamp copy(Timestamp value) {
+      return new Timestamp(value.getTime());
+    }
   }
 
-  public static final class LocalDateSerializer extends TimeSerializer<LocalDate> {
+  public static final class LocalDateSerializer extends ImmutableTimeSerializer<LocalDate> {
     public LocalDateSerializer(Fury fury) {
       super(fury, LocalDate.class);
     }
@@ -178,11 +211,6 @@ public class TimeSerializers {
       super(fury, LocalDate.class, needToWriteRef);
     }
 
-    @Override
-    public short getXtypeId() {
-      return Type.DATE32.getId();
-    }
-
     @Override
     public void xwrite(MemoryBuffer buffer, LocalDate value) {
       // TODO use java encoding to support larger range.
@@ -218,7 +246,7 @@ public class TimeSerializers {
     }
   }
 
-  public static final class InstantSerializer extends TimeSerializer<Instant> {
+  public static final class InstantSerializer extends ImmutableTimeSerializer<Instant> {
     public InstantSerializer(Fury fury) {
       super(fury, Instant.class);
     }
@@ -227,11 +255,6 @@ public class TimeSerializers {
       super(fury, Instant.class, needToWriteRef);
     }
 
-    @Override
-    public short getXtypeId() {
-      return Type.TIMESTAMP.getId();
-    }
-
     @Override
     public void xwrite(MemoryBuffer buffer, Instant value) {
       // FIXME JDK17 may have higher precision than millisecond
@@ -257,7 +280,7 @@ public class TimeSerializers {
     }
   }
 
-  public static class DurationSerializer extends TimeSerializer<Duration> {
+  public static class DurationSerializer extends ImmutableTimeSerializer<Duration> {
     public DurationSerializer(Fury fury) {
       super(fury, Duration.class);
     }
@@ -280,7 +303,7 @@ public class TimeSerializers {
     }
   }
 
-  public static class LocalDateTimeSerializer extends TimeSerializer<LocalDateTime> {
+  public static class LocalDateTimeSerializer extends ImmutableTimeSerializer<LocalDateTime> {
     public LocalDateTimeSerializer(Fury fury) {
       super(fury, LocalDateTime.class);
     }
@@ -303,7 +326,7 @@ public class TimeSerializers {
     }
   }
 
-  public static class LocalTimeSerializer extends TimeSerializer<LocalTime> {
+  public static class LocalTimeSerializer extends ImmutableTimeSerializer<LocalTime> {
     public LocalTimeSerializer(Fury fury) {
       super(fury, LocalTime.class);
     }
@@ -384,6 +407,11 @@ public class TimeSerializers {
     public TimeZone read(MemoryBuffer buffer) {
       return TimeZone.getTimeZone(fury.readJavaString(buffer));
     }
+
+    @Override
+    public TimeZone copy(TimeZone value) {
+      return TimeZone.getTimeZone(value.getID());
+    }
   }
 
   public static final class CalendarSerializer extends TimeSerializer<Calendar> {
@@ -427,9 +455,23 @@ public class TimeSerializers {
       }
       return result;
     }
+
+    @Override
+    public Calendar copy(Calendar value) {
+      Calendar copy = Calendar.getInstance(value.getTimeZone());
+      copy.setTimeInMillis(value.getTimeInMillis());
+      copy.setLenient(value.isLenient());
+      copy.setFirstDayOfWeek(value.getFirstDayOfWeek());
+      copy.setMinimalDaysInFirstWeek(value.getMinimalDaysInFirstWeek());
+      if (value instanceof GregorianCalendar) {
+        ((GregorianCalendar) copy)
+            .setGregorianChange(((GregorianCalendar) value).getGregorianChange());
+      }
+      return copy;
+    }
   }
 
-  public static class ZoneIdSerializer extends TimeSerializer<ZoneId> {
+  public static class ZoneIdSerializer extends ImmutableTimeSerializer<ZoneId> {
     public ZoneIdSerializer(Fury fury, Class<ZoneId> type) {
       super(fury, type);
     }
@@ -449,7 +491,7 @@ public class TimeSerializers {
     }
   }
 
-  public static class ZoneOffsetSerializer extends TimeSerializer<ZoneOffset> {
+  public static class ZoneOffsetSerializer extends ImmutableTimeSerializer<ZoneOffset> {
     public ZoneOffsetSerializer(Fury fury) {
       super(fury, ZoneOffset.class);
     }
@@ -483,7 +525,7 @@ public class TimeSerializers {
     }
   }
 
-  public static class ZonedDateTimeSerializer extends TimeSerializer<ZonedDateTime> {
+  public static class ZonedDateTimeSerializer extends ImmutableTimeSerializer<ZonedDateTime> {
 
     public ZonedDateTimeSerializer(Fury fury) {
       super(fury, ZonedDateTime.class);
@@ -507,7 +549,7 @@ public class TimeSerializers {
     }
   }
 
-  public static class YearSerializer extends TimeSerializer<Year> {
+  public static class YearSerializer extends ImmutableTimeSerializer<Year> {
     public YearSerializer(Fury fury) {
       super(fury, Year.class);
     }
@@ -527,7 +569,7 @@ public class TimeSerializers {
     }
   }
 
-  public static class YearMonthSerializer extends TimeSerializer<YearMonth> {
+  public static class YearMonthSerializer extends ImmutableTimeSerializer<YearMonth> {
     public YearMonthSerializer(Fury fury) {
       super(fury, YearMonth.class);
     }
@@ -550,7 +592,7 @@ public class TimeSerializers {
     }
   }
 
-  public static class MonthDaySerializer extends TimeSerializer<MonthDay> {
+  public static class MonthDaySerializer extends ImmutableTimeSerializer<MonthDay> {
     public MonthDaySerializer(Fury fury) {
       super(fury, MonthDay.class);
     }
@@ -571,7 +613,7 @@ public class TimeSerializers {
     }
   }
 
-  public static class PeriodSerializer extends TimeSerializer<Period> {
+  public static class PeriodSerializer extends ImmutableTimeSerializer<Period> {
     public PeriodSerializer(Fury fury) {
       super(fury, Period.class);
     }
@@ -594,7 +636,7 @@ public class TimeSerializers {
     }
   }
 
-  public static class OffsetTimeSerializer extends TimeSerializer<OffsetTime> {
+  public static class OffsetTimeSerializer extends ImmutableTimeSerializer<OffsetTime> {
     public OffsetTimeSerializer(Fury fury) {
       super(fury, OffsetTime.class);
     }
@@ -615,7 +657,7 @@ public class TimeSerializers {
     }
   }
 
-  public static class OffsetDateTimeSerializer extends TimeSerializer<OffsetDateTime> {
+  public static class OffsetDateTimeSerializer extends ImmutableTimeSerializer<OffsetDateTime> {
     public OffsetDateTimeSerializer(Fury fury) {
       super(fury, OffsetDateTime.class);
     }
@@ -639,22 +681,23 @@ public class TimeSerializers {
   }
 
   public static void registerDefaultSerializers(Fury fury) {
-    fury.registerSerializer(Date.class, new DateSerializer(fury));
-    fury.registerSerializer(java.sql.Date.class, new SqlDateSerializer(fury));
-    fury.registerSerializer(Time.class, new SqlTimeSerializer(fury));
-    fury.registerSerializer(Timestamp.class, new TimestampSerializer(fury));
-    fury.registerSerializer(LocalDate.class, new LocalDateSerializer(fury));
-    fury.registerSerializer(LocalTime.class, new LocalTimeSerializer(fury));
-    fury.registerSerializer(LocalDateTime.class, new LocalDateTimeSerializer(fury));
-    fury.registerSerializer(Instant.class, new InstantSerializer(fury));
-    fury.registerSerializer(Duration.class, new DurationSerializer(fury));
-    fury.registerSerializer(ZoneOffset.class, new ZoneOffsetSerializer(fury));
-    fury.registerSerializer(ZonedDateTime.class, new ZonedDateTimeSerializer(fury));
-    fury.registerSerializer(Year.class, new YearSerializer(fury));
-    fury.registerSerializer(YearMonth.class, new YearMonthSerializer(fury));
-    fury.registerSerializer(MonthDay.class, new MonthDaySerializer(fury));
-    fury.registerSerializer(Period.class, new PeriodSerializer(fury));
-    fury.registerSerializer(OffsetTime.class, new OffsetTimeSerializer(fury));
-    fury.registerSerializer(OffsetDateTime.class, new OffsetDateTimeSerializer(fury));
+    ClassResolver resolver = fury.getClassResolver();
+    resolver.registerSerializer(Date.class, new DateSerializer(fury));
+    resolver.registerSerializer(java.sql.Date.class, new SqlDateSerializer(fury));
+    resolver.registerSerializer(Time.class, new SqlTimeSerializer(fury));
+    resolver.registerSerializer(Timestamp.class, new TimestampSerializer(fury));
+    resolver.registerSerializer(LocalDate.class, new LocalDateSerializer(fury));
+    resolver.registerSerializer(LocalTime.class, new LocalTimeSerializer(fury));
+    resolver.registerSerializer(LocalDateTime.class, new LocalDateTimeSerializer(fury));
+    resolver.registerSerializer(Instant.class, new InstantSerializer(fury));
+    resolver.registerSerializer(Duration.class, new DurationSerializer(fury));
+    resolver.registerSerializer(ZoneOffset.class, new ZoneOffsetSerializer(fury));
+    resolver.registerSerializer(ZonedDateTime.class, new ZonedDateTimeSerializer(fury));
+    resolver.registerSerializer(Year.class, new YearSerializer(fury));
+    resolver.registerSerializer(YearMonth.class, new YearMonthSerializer(fury));
+    resolver.registerSerializer(MonthDay.class, new MonthDaySerializer(fury));
+    resolver.registerSerializer(Period.class, new PeriodSerializer(fury));
+    resolver.registerSerializer(OffsetTime.class, new OffsetTimeSerializer(fury));
+    resolver.registerSerializer(OffsetDateTime.class, new OffsetDateTimeSerializer(fury));
   }
 }
