@@ -58,7 +58,6 @@ import org.apache.fury.resolver.RefResolver;
 import org.apache.fury.serializer.ReplaceResolveSerializer;
 import org.apache.fury.serializer.Serializer;
 import org.apache.fury.serializer.Serializers;
-import org.apache.fury.type.Type;
 import org.apache.fury.util.Preconditions;
 import org.apache.fury.util.unsafe._JDKAccess;
 
@@ -74,11 +73,6 @@ public class CollectionSerializers {
       super(fury, ArrayList.class, true);
     }
 
-    @Override
-    public short getXtypeId() {
-      return Type.LIST.getId();
-    }
-
     @Override
     public ArrayList newCollection(MemoryBuffer buffer) {
       int numElements = buffer.readVarUint32Small7();
@@ -103,7 +97,7 @@ public class CollectionSerializers {
     }
 
     public ArraysAsListSerializer(Fury fury, Class<List<?>> cls) {
-      super(fury, cls, false);
+      super(fury, cls, fury.isCrossLanguage());
     }
 
     @Override
@@ -117,11 +111,6 @@ public class CollectionSerializers {
       return newCollection;
     }
 
-    @Override
-    public short getXtypeId() {
-      return (short) -Type.LIST.getId();
-    }
-
     @Override
     public void write(MemoryBuffer buffer, List<?> value) {
       try {
@@ -132,12 +121,6 @@ public class CollectionSerializers {
       }
     }
 
-    @Override
-    public void xwrite(MemoryBuffer buffer, List<?> value) {
-      // FIXME this may cause array data got duplicated when reference tracking enabled.
-      super.xwrite(buffer, value);
-    }
-
     @Override
     public List<?> read(MemoryBuffer buffer) {
       final Object[] array = (Object[]) fury.readRef(buffer);
@@ -145,15 +128,23 @@ public class CollectionSerializers {
       return Arrays.asList(array);
     }
 
+    @Override
+    public void xwrite(MemoryBuffer buffer, List<?> value) {
+      super.write(buffer, value);
+    }
+
     @Override
     public List<?> xread(MemoryBuffer buffer) {
+      return super.read(buffer);
+    }
+
+    @Override
+    public ArrayList newCollection(MemoryBuffer buffer) {
       int numElements = buffer.readVarUint32Small7();
-      Object[] arr = new Object[numElements];
-      for (int i = 0; i < numElements; i++) {
-        Object elem = fury.xreadRef(buffer);
-        arr[i] = elem;
-      }
-      return Arrays.asList(arr);
+      setNumElements(numElements);
+      ArrayList arrayList = new ArrayList(numElements);
+      fury.getRefResolver().reference(arrayList);
+      return arrayList;
     }
   }
 
@@ -162,11 +153,6 @@ public class CollectionSerializers {
       super(fury, HashSet.class, true);
     }
 
-    @Override
-    public short getXtypeId() {
-      return Type.FURY_SET.getId();
-    }
-
     @Override
     public HashSet newCollection(MemoryBuffer buffer) {
       int numElements = buffer.readVarUint32Small7();
@@ -182,11 +168,6 @@ public class CollectionSerializers {
       super(fury, LinkedHashSet.class, true);
     }
 
-    @Override
-    public short getXtypeId() {
-      return Type.FURY_SET.getId();
-    }
-
     @Override
     public LinkedHashSet newCollection(MemoryBuffer buffer) {
       int numElements = buffer.readVarUint32Small7();
@@ -214,13 +195,16 @@ public class CollectionSerializers {
     @Override
     public Collection onCollectionWrite(MemoryBuffer buffer, T value) {
       buffer.writeVarUint32Small7(value.size());
-      fury.writeRef(buffer, value.comparator());
+      if (!fury.isCrossLanguage()) {
+        fury.writeRef(buffer, value.comparator());
+      }
       return value;
     }
 
     @SuppressWarnings("unchecked")
     @Override
     public T newCollection(MemoryBuffer buffer) {
+      assert !fury.isCrossLanguage();
       int numElements = buffer.readVarUint32Small7();
       setNumElements(numElements);
       T collection;
@@ -269,11 +253,6 @@ public class CollectionSerializers {
     @Override
     public void write(MemoryBuffer buffer, List<?> value) {}
 
-    @Override
-    public short getXtypeId() {
-      return (short) -Type.LIST.getId();
-    }
-
     @Override
     public void xwrite(MemoryBuffer buffer, List<?> value) {
       // write length
@@ -316,21 +295,15 @@ public class CollectionSerializers {
   public static final class EmptySetSerializer extends CollectionSerializer<Set<?>> {
 
     public EmptySetSerializer(Fury fury, Class<Set<?>> cls) {
-      super(fury, cls, false, true);
+      super(fury, cls, fury.isCrossLanguage(), true);
     }
 
     @Override
     public void write(MemoryBuffer buffer, Set<?> value) {}
 
-    @Override
-    public short getXtypeId() {
-      return (short) -Type.FURY_SET.getId();
-    }
-
     @Override
     public void xwrite(MemoryBuffer buffer, Set<?> value) {
-      // write length
-      buffer.writeVarUint32Small7(0);
+      super.write(buffer, value);
     }
 
     @Override
@@ -340,15 +313,14 @@ public class CollectionSerializers {
 
     @Override
     public Set<?> xread(MemoryBuffer buffer) {
-      buffer.readVarUint32Small7();
-      return Collections.EMPTY_SET;
+      throw new UnsupportedOperationException();
     }
   }
 
   public static final class EmptySortedSetSerializer extends CollectionSerializer<SortedSet<?>> {
 
     public EmptySortedSetSerializer(Fury fury, Class<SortedSet<?>> cls) {
-      super(fury, cls, false, true);
+      super(fury, cls, fury.isCrossLanguage(), true);
     }
 
     @Override
@@ -358,13 +330,23 @@ public class CollectionSerializers {
     public SortedSet<?> read(MemoryBuffer buffer) {
       return Collections.emptySortedSet();
     }
+
+    @Override
+    public void xwrite(MemoryBuffer buffer, SortedSet<?> value) {
+      super.write(buffer, value);
+    }
+
+    @Override
+    public SortedSet<?> xread(MemoryBuffer buffer) {
+      throw new UnsupportedOperationException();
+    }
   }
 
   public static final class CollectionsSingletonListSerializer
       extends CollectionSerializer<List<?>> {
 
     public CollectionsSingletonListSerializer(Fury fury, Class<List<?>> cls) {
-      super(fury, cls, false);
+      super(fury, cls, fury.isCrossLanguage());
     }
 
     @Override
@@ -377,15 +359,9 @@ public class CollectionSerializers {
       fury.writeRef(buffer, value.get(0));
     }
 
-    @Override
-    public short getXtypeId() {
-      return (short) -Type.LIST.getId();
-    }
-
     @Override
     public void xwrite(MemoryBuffer buffer, List<?> value) {
-      buffer.writeVarUint32Small7(1);
-      fury.xwriteRef(buffer, value.get(0));
+      super.write(buffer, value);
     }
 
     @Override
@@ -395,15 +371,14 @@ public class CollectionSerializers {
 
     @Override
     public List<?> xread(MemoryBuffer buffer) {
-      buffer.readVarUint32Small7();
-      return Collections.singletonList(fury.xreadRef(buffer));
+      throw new UnsupportedOperationException();
     }
   }
 
   public static final class CollectionsSingletonSetSerializer extends CollectionSerializer<Set<?>> {
 
     public CollectionsSingletonSetSerializer(Fury fury, Class<Set<?>> cls) {
-      super(fury, cls, false);
+      super(fury, cls, fury.isCrossLanguage());
     }
 
     @Override
@@ -416,15 +391,9 @@ public class CollectionSerializers {
       fury.writeRef(buffer, value.iterator().next());
     }
 
-    @Override
-    public short getXtypeId() {
-      return (short) -Type.FURY_SET.getId();
-    }
-
     @Override
     public void xwrite(MemoryBuffer buffer, Set<?> value) {
-      buffer.writeVarUint32Small7(1);
-      fury.xwriteRef(buffer, value.iterator().next());
+      super.write(buffer, value);
     }
 
     @Override
@@ -434,8 +403,7 @@ public class CollectionSerializers {
 
     @Override
     public Set<?> xread(MemoryBuffer buffer) {
-      buffer.readVarUint32Small7();
-      return Collections.singleton(fury.xreadRef(buffer));
+      throw new UnsupportedOperationException();
     }
   }
 
@@ -450,6 +418,7 @@ public class CollectionSerializers {
     public ConcurrentSkipListSet newCollection(MemoryBuffer buffer) {
       int numElements = buffer.readVarUint32Small7();
       setNumElements(numElements);
+      assert !fury.isCrossLanguage();
       RefResolver refResolver = fury.getRefResolver();
       int refId = refResolver.lastPreservedRefId();
       // It's possible that comparator/elements has circular ref to set.
@@ -516,6 +485,7 @@ public class CollectionSerializers {
 
     @Override
     public Collection newCollection(Collection originCollection) {
+      assert !fury.isCrossLanguage();
       Map<?, Boolean> map =
           (Map<?, Boolean>) Platform.getObject(originCollection, MAP_FIELD_OFFSET);
       AbstractMapSerializer mapSerializer =
@@ -529,7 +499,7 @@ public class CollectionSerializers {
       final Map<?, Boolean> map = (Map<?, Boolean>) Platform.getObject(value, MAP_FIELD_OFFSET);
       final ClassInfo classInfo = fury.getClassResolver().getClassInfo(map.getClass());
       AbstractMapSerializer mapSerializer = (AbstractMapSerializer) classInfo.getSerializer();
-      fury.getClassResolver().writeClass(buffer, classInfo);
+      fury.getClassResolver().writeClassInfo(buffer, classInfo);
       if (mapSerializer.supportCodegenHook) {
         buffer.writeBoolean(true);
         mapSerializer.onMapWrite(buffer, map);
@@ -688,7 +658,9 @@ public class CollectionSerializers {
 
     public Collection onCollectionWrite(MemoryBuffer buffer, PriorityQueue value) {
       buffer.writeVarUint32Small7(value.size());
-      fury.writeRef(buffer, value.comparator());
+      if (!fury.isCrossLanguage()) {
+        fury.writeRef(buffer, value.comparator());
+      }
       return value;
     }
 
@@ -700,6 +672,7 @@ public class CollectionSerializers {
 
     @Override
     public PriorityQueue newCollection(MemoryBuffer buffer) {
+      assert !fury.isCrossLanguage();
       int numElements = buffer.readVarUint32Small7();
       setNumElements(numElements);
       Comparator comparator = (Comparator) fury.readRef(buffer);
@@ -807,46 +780,48 @@ public class CollectionSerializers {
   // TODO Support ArraySubListSerializer, SubListSerializer
 
   public static void registerDefaultSerializers(Fury fury) {
-    fury.registerSerializer(ArrayList.class, new ArrayListSerializer(fury));
+    ClassResolver resolver = fury.getClassResolver();
+    resolver.registerSerializer(ArrayList.class, new ArrayListSerializer(fury));
     Class arrayAsListClass = Arrays.asList(1, 2).getClass();
-    fury.registerSerializer(arrayAsListClass, new ArraysAsListSerializer(fury, arrayAsListClass));
-    fury.registerSerializer(
+    resolver.registerSerializer(
+        arrayAsListClass, new ArraysAsListSerializer(fury, arrayAsListClass));
+    resolver.registerSerializer(
         LinkedList.class, new CollectionSerializer(fury, LinkedList.class, true));
-    fury.registerSerializer(HashSet.class, new HashSetSerializer(fury));
-    fury.registerSerializer(LinkedHashSet.class, new LinkedHashSetSerializer(fury));
-    fury.registerSerializer(TreeSet.class, new SortedSetSerializer<>(fury, TreeSet.class));
-    fury.registerSerializer(
+    resolver.registerSerializer(HashSet.class, new HashSetSerializer(fury));
+    resolver.registerSerializer(LinkedHashSet.class, new LinkedHashSetSerializer(fury));
+    resolver.registerSerializer(TreeSet.class, new SortedSetSerializer<>(fury, TreeSet.class));
+    resolver.registerSerializer(
         Collections.EMPTY_LIST.getClass(),
         new EmptyListSerializer(fury, (Class<List<?>>) Collections.EMPTY_LIST.getClass()));
-    fury.registerSerializer(
+    resolver.registerSerializer(
         Collections.emptySortedSet().getClass(),
         new EmptySortedSetSerializer(
             fury, (Class<SortedSet<?>>) Collections.emptySortedSet().getClass()));
-    fury.registerSerializer(
+    resolver.registerSerializer(
         Collections.EMPTY_SET.getClass(),
         new EmptySetSerializer(fury, (Class<Set<?>>) Collections.EMPTY_SET.getClass()));
-    fury.registerSerializer(
+    resolver.registerSerializer(
         Collections.singletonList(null).getClass(),
         new CollectionsSingletonListSerializer(
             fury, (Class<List<?>>) Collections.singletonList(null).getClass()));
-    fury.registerSerializer(
+    resolver.registerSerializer(
         Collections.singleton(null).getClass(),
         new CollectionsSingletonSetSerializer(
             fury, (Class<Set<?>>) Collections.singleton(null).getClass()));
-    fury.registerSerializer(
+    resolver.registerSerializer(
         ConcurrentSkipListSet.class,
         new ConcurrentSkipListSetSerializer(fury, ConcurrentSkipListSet.class));
-    fury.registerSerializer(Vector.class, new VectorSerializer(fury, Vector.class));
-    fury.registerSerializer(ArrayDeque.class, new ArrayDequeSerializer(fury, ArrayDeque.class));
-    fury.registerSerializer(BitSet.class, new BitSetSerializer(fury, BitSet.class));
-    fury.registerSerializer(
+    resolver.registerSerializer(Vector.class, new VectorSerializer(fury, Vector.class));
+    resolver.registerSerializer(ArrayDeque.class, new ArrayDequeSerializer(fury, ArrayDeque.class));
+    resolver.registerSerializer(BitSet.class, new BitSetSerializer(fury, BitSet.class));
+    resolver.registerSerializer(
         PriorityQueue.class, new PriorityQueueSerializer(fury, PriorityQueue.class));
-    fury.registerSerializer(
+    resolver.registerSerializer(
         CopyOnWriteArrayList.class,
         new CopyOnWriteArrayListSerializer(fury, CopyOnWriteArrayList.class));
     final Class setFromMapClass = Collections.newSetFromMap(new HashMap<>()).getClass();
-    fury.registerSerializer(setFromMapClass, new SetFromMapSerializer(fury, setFromMapClass));
-    fury.registerSerializer(
+    resolver.registerSerializer(setFromMapClass, new SetFromMapSerializer(fury, setFromMapClass));
+    resolver.registerSerializer(
         ConcurrentHashMap.KeySetView.class,
         new ConcurrentHashMapKeySetViewSerializer(fury, ConcurrentHashMap.KeySetView.class));
   }
