@@ -37,12 +37,12 @@ import org.apache.fury.config.Language;
 import org.apache.fury.memory.MemoryBuffer;
 import org.apache.fury.memory.Platform;
 import org.apache.fury.reflect.ReflectionUtils;
+import org.apache.fury.resolver.ClassInfo;
 import org.apache.fury.resolver.ClassResolver;
 import org.apache.fury.serializer.ReplaceResolveSerializer;
 import org.apache.fury.serializer.Serializer;
 import org.apache.fury.serializer.Serializers;
 import org.apache.fury.serializer.StringSerializer;
-import org.apache.fury.type.Type;
 import org.apache.fury.util.Preconditions;
 
 /**
@@ -57,11 +57,6 @@ public class MapSerializers {
       super(fury, HashMap.class, true);
     }
 
-    @Override
-    public short getXtypeId() {
-      return Type.MAP.getId();
-    }
-
     @Override
     public HashMap newMap(MemoryBuffer buffer) {
       int numElements = buffer.readVarUint32Small7();
@@ -82,11 +77,6 @@ public class MapSerializers {
       super(fury, LinkedHashMap.class, true);
     }
 
-    @Override
-    public short getXtypeId() {
-      return Type.MAP.getId();
-    }
-
     @Override
     public LinkedHashMap newMap(MemoryBuffer buffer) {
       int numElements = buffer.readVarUint32Small7();
@@ -107,11 +97,6 @@ public class MapSerializers {
       super(fury, LazyMap.class, true);
     }
 
-    @Override
-    public short getXtypeId() {
-      return Type.MAP.getId();
-    }
-
     @Override
     public LazyMap newMap(MemoryBuffer buffer) {
       int numElements = buffer.readVarUint32Small7();
@@ -139,13 +124,16 @@ public class MapSerializers {
     @Override
     public Map onMapWrite(MemoryBuffer buffer, T value) {
       buffer.writeVarUint32Small7(value.size());
-      fury.writeRef(buffer, value.comparator());
+      if (!fury.isCrossLanguage()) {
+        fury.writeRef(buffer, value.comparator());
+      }
       return value;
     }
 
     @SuppressWarnings("unchecked")
     @Override
     public Map newMap(MemoryBuffer buffer) {
+      assert !fury.isCrossLanguage();
       setNumElements(buffer.readVarUint32Small7());
       T map;
       Comparator comparator = (Comparator) fury.readRef(buffer);
@@ -182,21 +170,15 @@ public class MapSerializers {
   public static final class EmptyMapSerializer extends MapSerializer<Map<?, ?>> {
 
     public EmptyMapSerializer(Fury fury, Class<Map<?, ?>> cls) {
-      super(fury, cls, false, true);
+      super(fury, cls, fury.isCrossLanguage(), true);
     }
 
     @Override
     public void write(MemoryBuffer buffer, Map<?, ?> value) {}
 
-    @Override
-    public short getXtypeId() {
-      return (short) -Type.MAP.getId();
-    }
-
     @Override
     public void xwrite(MemoryBuffer buffer, Map<?, ?> value) {
-      // write length
-      buffer.writeVarUint32Small7(0);
+      super.write(buffer, value);
     }
 
     @Override
@@ -206,14 +188,13 @@ public class MapSerializers {
 
     @Override
     public Map<?, ?> xread(MemoryBuffer buffer) {
-      buffer.readVarUint32Small7();
-      return Collections.EMPTY_MAP;
+      throw new IllegalStateException();
     }
   }
 
   public static final class EmptySortedMapSerializer extends MapSerializer<SortedMap<?, ?>> {
     public EmptySortedMapSerializer(Fury fury, Class<SortedMap<?, ?>> cls) {
-      super(fury, cls, false, true);
+      super(fury, cls, fury.isCrossLanguage(), true);
     }
 
     @Override
@@ -228,7 +209,7 @@ public class MapSerializers {
   public static final class SingletonMapSerializer extends MapSerializer<Map<?, ?>> {
 
     public SingletonMapSerializer(Fury fury, Class<Map<?, ?>> cls) {
-      super(fury, cls, false);
+      super(fury, cls, fury.isCrossLanguage());
     }
 
     @Override
@@ -245,19 +226,6 @@ public class MapSerializers {
       fury.writeRef(buffer, entry.getValue());
     }
 
-    @Override
-    public short getXtypeId() {
-      return (short) -Type.MAP.getId();
-    }
-
-    @Override
-    public void xwrite(MemoryBuffer buffer, Map<?, ?> value) {
-      buffer.writeVarUint32Small7(1);
-      Map.Entry entry = value.entrySet().iterator().next();
-      fury.xwriteRef(buffer, entry.getKey());
-      fury.xwriteRef(buffer, entry.getValue());
-    }
-
     @Override
     public Map<?, ?> read(MemoryBuffer buffer) {
       Object key = fury.readRef(buffer);
@@ -265,12 +233,14 @@ public class MapSerializers {
       return Collections.singletonMap(key, value);
     }
 
+    @Override
+    public void xwrite(MemoryBuffer buffer, Map<?, ?> value) {
+      super.write(buffer, value);
+    }
+
     @Override
     public Map<?, ?> xread(MemoryBuffer buffer) {
-      buffer.readVarUint32Small7();
-      Object key = fury.xreadRef(buffer);
-      Object value = fury.xreadRef(buffer);
-      return Collections.singletonMap(key, value);
+      throw new UnsupportedOperationException();
     }
   }
 
@@ -293,11 +263,6 @@ public class MapSerializers {
     public Map newMap(Map map) {
       return new ConcurrentHashMap(map.size());
     }
-
-    @Override
-    public short getXtypeId() {
-      return Fury.NOT_SUPPORT_CROSS_LANGUAGE;
-    }
   }
 
   public static final class ConcurrentSkipListMapSerializer
@@ -322,11 +287,6 @@ public class MapSerializers {
       Comparator comparator = fury.copyObject(((ConcurrentSkipListMap) originMap).comparator());
       return new ConcurrentSkipListMap(comparator);
     }
-
-    @Override
-    public short getXtypeId() {
-      return Fury.NOT_SUPPORT_CROSS_LANGUAGE;
-    }
   }
 
   public static class EnumMapSerializer extends MapSerializer<EnumMap> {
@@ -398,8 +358,17 @@ public class MapSerializers {
 
     @Override
     protected <K, V> void copyEntry(Map<K, V> originMap, Map<K, V> newMap) {
+      ClassResolver classResolver = fury.getClassResolver();
       for (Entry<K, V> entry : originMap.entrySet()) {
-        newMap.put(entry.getKey(), fury.copyObject(entry.getValue()));
+        V value = entry.getValue();
+        if (value != null) {
+          ClassInfo classInfo =
+              classResolver.getClassInfo(value.getClass(), valueClassInfoWriteCache);
+          if (!classInfo.getSerializer().isImmutable()) {
+            value = fury.copyObject(value, classInfo.getClassId());
+          }
+        }
+        newMap.put(entry.getKey(), value);
       }
     }
   }
@@ -507,27 +476,28 @@ public class MapSerializers {
 
   // TODO(chaokunyang) support ConcurrentSkipListMap.SubMap mo efficiently.
   public static void registerDefaultSerializers(Fury fury) {
-    fury.registerSerializer(HashMap.class, new HashMapSerializer(fury));
+    ClassResolver resolver = fury.getClassResolver();
+    resolver.registerSerializer(HashMap.class, new HashMapSerializer(fury));
     fury.getClassResolver()
         .registerSerializer(LinkedHashMap.class, new LinkedHashMapSerializer(fury));
-    fury.registerSerializer(TreeMap.class, new SortedMapSerializer<>(fury, TreeMap.class));
-    fury.registerSerializer(
+    resolver.registerSerializer(TreeMap.class, new SortedMapSerializer<>(fury, TreeMap.class));
+    resolver.registerSerializer(
         Collections.EMPTY_MAP.getClass(),
         new EmptyMapSerializer(fury, (Class<Map<?, ?>>) Collections.EMPTY_MAP.getClass()));
-    fury.registerSerializer(
+    resolver.registerSerializer(
         Collections.emptySortedMap().getClass(),
         new EmptySortedMapSerializer(
             fury, (Class<SortedMap<?, ?>>) Collections.emptySortedMap().getClass()));
-    fury.registerSerializer(
+    resolver.registerSerializer(
         Collections.singletonMap(null, null).getClass(),
         new SingletonMapSerializer(
             fury, (Class<Map<?, ?>>) Collections.singletonMap(null, null).getClass()));
-    fury.registerSerializer(
+    resolver.registerSerializer(
         ConcurrentHashMap.class, new ConcurrentHashMapSerializer(fury, ConcurrentHashMap.class));
-    fury.registerSerializer(
+    resolver.registerSerializer(
         ConcurrentSkipListMap.class,
         new ConcurrentSkipListMapSerializer(fury, ConcurrentSkipListMap.class));
-    fury.registerSerializer(EnumMap.class, new EnumMapSerializer(fury));
-    fury.registerSerializer(LazyMap.class, new LazyMapSerializer(fury));
+    resolver.registerSerializer(EnumMap.class, new EnumMapSerializer(fury));
+    resolver.registerSerializer(LazyMap.class, new LazyMapSerializer(fury));
   }
 }
