@@ -42,6 +42,7 @@ import java.util.concurrent.ExecutionException;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.ForkJoinPool;
 import java.util.concurrent.atomic.AtomicBoolean;
+import org.apache.fury.annotation.Expose;
 import org.apache.fury.annotation.Ignore;
 import org.apache.fury.annotation.Internal;
 import org.apache.fury.collection.Tuple2;
@@ -77,6 +78,7 @@ public class Descriptor {
   private Class<?> type;
   private final String typeName;
   private final String name;
+  private String snakeCaseName;
   private final int modifier;
   private final String declaringClass;
   private final Field field;
@@ -153,6 +155,13 @@ public class Descriptor {
     return name;
   }
 
+  public String getSnakeCaseName() {
+    if (snakeCaseName == null) {
+      snakeCaseName = StringUtils.lowerCamelToLowerUnderscore(name);
+    }
+    return snakeCaseName;
+  }
+
   public int getModifiers() {
     return modifier;
   }
@@ -205,7 +214,7 @@ public class Descriptor {
     sb.append(", name=").append(name);
     sb.append(", modifier=").append(modifier);
     if (field != null) {
-      sb.append(", field=").append(field.getDeclaringClass().getSimpleName()).append('.');
+      sb.append(", declaringClass=").append(field.getDeclaringClass().getSimpleName());
     }
     if (readMethod != null) {
       sb.append(", readMethod=").append(readMethod);
@@ -370,14 +379,33 @@ public class Descriptor {
     }
     do {
       Field[] fields = clazz.getDeclaredFields();
+      boolean haveExpose = false, haveIgnore = false;
       for (Field field : fields) {
         warmField(clz, field, compilationService);
+        if (field.isAnnotationPresent(Expose.class)) {
+          haveExpose = true;
+        }
+        if (field.isAnnotationPresent(Ignore.class)) {
+          haveIgnore = true;
+        }
+        if (haveExpose && haveIgnore) {
+          throw new RuntimeException(
+              "Fields of a Class are not allowed to have both the Ignore and Expose annotations simultaneously.");
+        }
+      }
+      for (Field field : fields) {
         int modifiers = field.getModifiers();
         // final and non-private field validation left to {@link isBean(clz)}
-        if (!Modifier.isTransient(modifiers)
-            && !Modifier.isStatic(modifiers)
-            && !field.isAnnotationPresent(Ignore.class)) {
-          descriptorMap.put(field, new Descriptor(field, null));
+        if (!Modifier.isTransient(modifiers) && !Modifier.isStatic(modifiers)) {
+          if (haveExpose) {
+            if (field.isAnnotationPresent(Expose.class)) {
+              descriptorMap.put(field, new Descriptor(field, null));
+            }
+          } else {
+            if (!field.isAnnotationPresent(Ignore.class)) {
+              descriptorMap.put(field, new Descriptor(field, null));
+            }
+          }
         }
       }
       if (clazz == clz) {
