@@ -19,6 +19,7 @@
 
 package org.apache.fury.serializer;
 
+import static org.apache.fury.reflect.ReflectionUtils.getObjectFieldValue;
 import static org.apache.fury.serializer.ClassUtils.loadClass;
 
 import com.google.common.collect.ImmutableSet;
@@ -31,6 +32,8 @@ import java.util.stream.Collectors;
 import org.apache.fury.Fury;
 import org.apache.fury.FuryTestBase;
 import org.apache.fury.builder.MetaSharedCodecBuilder;
+import org.apache.fury.codegen.CompileUnit;
+import org.apache.fury.codegen.JaninoUtils;
 import org.apache.fury.config.CompatibleMode;
 import org.apache.fury.config.FuryBuilder;
 import org.apache.fury.config.Language;
@@ -52,7 +55,7 @@ import org.testng.annotations.Test;
  * interoperability between them.
  */
 public class MetaSharedCompatibleTest extends FuryTestBase {
-  public static Object serDeCheck(Fury fury, Object obj) {
+  public static Object serDeMetaSharedCheck(Fury fury, Object obj) {
     Object newObj = serDeMetaShared(fury, obj);
     Assert.assertEquals(newObj, obj);
     return newObj;
@@ -113,9 +116,9 @@ public class MetaSharedCompatibleTest extends FuryTestBase {
             .withRefTracking(referenceTracking)
             .withCodegen(enableCodegen)
             .build();
-    serDeCheck(fury, Foo.create());
-    serDeCheck(fury, BeanB.createBeanB(2));
-    serDeCheck(fury, BeanA.createBeanA(2));
+    serDeMetaSharedCheck(fury, Foo.create());
+    serDeMetaSharedCheck(fury, BeanB.createBeanB(2));
+    serDeMetaSharedCheck(fury, BeanA.createBeanA(2));
   }
 
   @Test(dataProvider = "config2")
@@ -671,6 +674,210 @@ public class MetaSharedCompatibleTest extends FuryTestBase {
         new ClassDefEncoderTest
             .TestClassLengthTestClassLengthTestClassLengthTestClassLengthTestClassLengthTestClassLengthTestClassLength
             .InnerClassTestLengthInnerClassTestLengthInnerClassTestLength();
-    serDeCheck(fury, o);
+    serDeMetaSharedCheck(fury, o);
+  }
+
+  CompileUnit aunit =
+      new CompileUnit(
+          "demo.pkg1",
+          "A",
+          (""
+              + "package demo.pkg1;\n"
+              + "import demo.test.*;\n"
+              + "import demo.report.*;\n"
+              + "public class A {\n"
+              + "  int f1;\n"
+              + "  B f2;\n"
+              + "  C f3;\n"
+              + "  public String toString() {return \"A\" + \",\" + f1 + \",\" + f2 + \",\" + f3;}\n"
+              + "  public static A create() { A a = new A(); a.f1 = 10; a.f2 = new B(); a.f3 = new C(); return a;}\n"
+              + "}"));
+
+  CompileUnit bunit =
+      new CompileUnit(
+          "demo.test",
+          "B",
+          ("" + "package demo.test;\n" + "public class B {\n" + "  public int f1 = 100;\n" + "}"));
+
+  CompileUnit cunit =
+      new CompileUnit(
+          "demo.report",
+          "C",
+          (""
+              + "package demo.report;\n"
+              + "public class C {\n"
+              + "  public int f2 = 1000;\n"
+              + "  public String toString() {return \"C{f2=\" + f2 + \"}\";}\n"
+              + "}"));
+
+  CompileUnit newAUnit =
+      new CompileUnit(
+          "example.pkg1",
+          "A",
+          (""
+              + "package example.pkg1;\n"
+              + "import example.test.*;\n"
+              + "import example.report.*;\n"
+              + "public class A {\n"
+              + "  public int f1;\n"
+              + "  public C f3;\n"
+              + "  public String toString() {return \"A\" + \",\" + f1 + \",\" + f3;}\n"
+              + "  public static A create() { A a = new A(); a.f1 = 10; a.f3 = new C(); return a;}\n"
+              + "}"));
+
+  CompileUnit newCUnit =
+      new CompileUnit(
+          "example.test",
+          "C",
+          (""
+              + "package example.test;\n"
+              + "public class C {\n"
+              + "  public int f2;\n"
+              + "  public String toString() {return \"C{f2=\" + f2 + \"}\";}\n"
+              + "}"));
+
+  @Test
+  public void testRegisterToSameIdForRenamedClass() throws Exception {
+    ClassLoader classLoader =
+        JaninoUtils.compile(Thread.currentThread().getContextClassLoader(), aunit, bunit, cunit);
+    byte[] serialized;
+    {
+      Class<?> A = classLoader.loadClass("demo.pkg1.A");
+      Class<?> B = classLoader.loadClass("demo.test.B");
+      Class<?> C = classLoader.loadClass("demo.report.C");
+      Fury fury =
+          builder()
+              .withCompatibleMode(CompatibleMode.COMPATIBLE)
+              .withClassLoader(classLoader)
+              .build();
+      fury.register(A);
+      fury.register(B);
+      fury.register(C);
+      Object a = A.getMethod("create").invoke(null);
+      System.out.println(a);
+      serialized = fury.serialize(a);
+    }
+    {
+      CompileUnit unit1 =
+          new CompileUnit(
+              "example.pkg1",
+              "A",
+              (""
+                  + "package example.pkg1;\n"
+                  + "import example.test.*;\n"
+                  + "import example.report.*;\n"
+                  + "public class A {\n"
+                  + "  int f1;\n"
+                  + "  B f2;\n"
+                  + "  C f3;\n"
+                  + "  public String toString() {return \"A\" + \",\" + f1 + \",\" + f2 + \",\" + f3;}\n"
+                  + "  public static A create() { A a = new A(); a.f1 = 10; a.f2 = new B(); a.f3 = new C(); return a;}\n"
+                  + "}"));
+      CompileUnit unit2 =
+          new CompileUnit(
+              "example.report",
+              "B",
+              ("" + "package example.report;\n" + "public class B {\n" + "}"));
+      CompileUnit unit3 =
+          new CompileUnit(
+              "example.test", "C", ("" + "package example.test;\n" + "public class C {\n" + "}"));
+      classLoader =
+          JaninoUtils.compile(Thread.currentThread().getContextClassLoader(), unit1, unit2, unit3);
+      Class<?> A = classLoader.loadClass("example.pkg1.A");
+      Class<?> B = classLoader.loadClass("example.report.B");
+      Class<?> C = classLoader.loadClass("example.test.C");
+      Fury fury =
+          builder()
+              .withCompatibleMode(CompatibleMode.COMPATIBLE)
+              .withClassLoader(classLoader)
+              .build();
+      fury.register(A);
+      fury.register(B);
+      fury.register(C);
+      Object newObj = fury.deserialize(serialized);
+      System.out.println(newObj);
+    }
+  }
+
+  @Test
+  public void testInconsistentRegistrationID() throws Exception {
+    ClassLoader classLoader =
+        JaninoUtils.compile(Thread.currentThread().getContextClassLoader(), aunit, bunit, cunit);
+    byte[] serialized;
+    {
+      Class<?> A = classLoader.loadClass("demo.pkg1.A");
+      Class<?> B = classLoader.loadClass("demo.test.B");
+      Class<?> C = classLoader.loadClass("demo.report.C");
+      Fury fury =
+          builder()
+              .withCompatibleMode(CompatibleMode.COMPATIBLE)
+              .withClassLoader(classLoader)
+              .build();
+      fury.register(A, 300);
+      fury.register(B, 301);
+      fury.register(C, 302);
+      Object a = A.getMethod("create").invoke(null);
+      System.out.println(a);
+      serialized = fury.serialize(a);
+    }
+    {
+      classLoader =
+          JaninoUtils.compile(Thread.currentThread().getContextClassLoader(), newAUnit, newCUnit);
+      Class<?> A = classLoader.loadClass("example.pkg1.A");
+      Class<?> C = classLoader.loadClass("example.test.C");
+      Fury fury =
+          builder()
+              .withCompatibleMode(CompatibleMode.COMPATIBLE)
+              .withClassLoader(classLoader)
+              .build();
+      fury.register(A, 300);
+      fury.register(C, 302);
+      Object newObj = fury.deserialize(serialized);
+      System.out.println(newObj);
+      Object f3 = getObjectFieldValue(newObj, "f3");
+      Assert.assertNotNull(f3);
+      Assert.assertEquals(f3.toString(), "C{f2=1000}");
+    }
+  }
+
+  @Test
+  public void testInconsistentRegistrationName() throws Exception {
+    ClassLoader classLoader =
+        JaninoUtils.compile(Thread.currentThread().getContextClassLoader(), aunit, bunit, cunit);
+    byte[] serialized;
+    {
+      Class<?> A = classLoader.loadClass("demo.pkg1.A");
+      Class<?> B = classLoader.loadClass("demo.test.B");
+      Class<?> C = classLoader.loadClass("demo.report.C");
+      Fury fury =
+          builder()
+              .withCompatibleMode(CompatibleMode.COMPATIBLE)
+              .withClassLoader(classLoader)
+              .build();
+      fury.register(A, "test.A");
+      fury.register(B, "test.B");
+      fury.register(C, "test.C");
+      Object a = A.getMethod("create").invoke(null);
+      System.out.println(a);
+      serialized = fury.serialize(a);
+    }
+    {
+      classLoader =
+          JaninoUtils.compile(Thread.currentThread().getContextClassLoader(), newAUnit, newCUnit);
+      Class<?> A = classLoader.loadClass("example.pkg1.A");
+      Class<?> C = classLoader.loadClass("example.test.C");
+      Fury fury =
+          builder()
+              .withCompatibleMode(CompatibleMode.COMPATIBLE)
+              .withClassLoader(classLoader)
+              .build();
+      fury.register(A, "test.A");
+      fury.register(C, "test.C");
+      Object newObj = fury.deserialize(serialized);
+      System.out.println(newObj);
+      Object f3 = getObjectFieldValue(newObj, "f3");
+      Assert.assertNotNull(f3);
+      Assert.assertEquals(f3.toString(), "C{f2=1000}");
+    }
   }
 }
