@@ -36,259 +36,268 @@ import org.apache.arrow.vector.ipc.message.MessageSerializer;
 import org.apache.arrow.vector.types.DateUnit;
 import org.apache.arrow.vector.types.FloatingPointPrecision;
 import org.apache.arrow.vector.types.TimeUnit;
-import org.apache.arrow.vector.types.pojo.ArrowType;
 import org.apache.arrow.vector.types.pojo.Field;
 import org.apache.arrow.vector.types.pojo.FieldType;
 import org.apache.arrow.vector.types.pojo.Schema;
 import org.apache.fury.exception.FuryException;
 import org.apache.fury.io.MemoryBufferOutputStream;
 import org.apache.fury.memory.MemoryBuffer;
-import org.apache.fury.type.Type;
 import org.apache.fury.util.DecimalUtils;
 import org.apache.fury.util.Preconditions;
 
 /** Arrow data type utils. */
 public class DataTypes {
-  public static Field PRIMITIVE_BOOLEAN_ARRAY_FIELD = primitiveArrayField(ArrowType.Bool.INSTANCE);
+  public static Field PRIMITIVE_BOOLEAN_ARRAY_FIELD =
+      primitiveArrayField(org.apache.arrow.vector.types.pojo.ArrowType.Bool.INSTANCE);
   public static Field PRIMITIVE_BYTE_ARRAY_FIELD = primitiveArrayField(intType(8));
   public static Field PRIMITIVE_SHORT_ARRAY_FIELD = primitiveArrayField(intType(16));
   public static Field PRIMITIVE_INT_ARRAY_FIELD = primitiveArrayField(intType(32));
 
   public static Field PRIMITIVE_LONG_ARRAY_FIELD = primitiveArrayField(intType(64));
   public static Field PRIMITIVE_FLOAT_ARRAY_FIELD =
-      primitiveArrayField(new ArrowType.FloatingPoint(FloatingPointPrecision.SINGLE));
+      primitiveArrayField(
+          new org.apache.arrow.vector.types.pojo.ArrowType.FloatingPoint(
+              FloatingPointPrecision.SINGLE));
   public static Field PRIMITIVE_DOUBLE_ARRAY_FIELD =
-      primitiveArrayField(new ArrowType.FloatingPoint(FloatingPointPrecision.DOUBLE));
+      primitiveArrayField(
+          new org.apache.arrow.vector.types.pojo.ArrowType.FloatingPoint(
+              FloatingPointPrecision.DOUBLE));
   // Array item field default name
   public static final String ARRAY_ITEM_NAME = "item";
 
-  private static final ArrowType.ArrowTypeVisitor<Integer> typeWidthVisitor =
-      new DefaultTypeVisitor<Integer>() {
+  private static final org.apache.arrow.vector.types.pojo.ArrowType.ArrowTypeVisitor<Integer>
+      typeWidthVisitor =
+          new DefaultTypeVisitor<Integer>() {
 
-        @Override
-        public Integer visit(ArrowType.Struct type) {
-          return -1;
-        }
-
-        @Override
-        public Integer visit(ArrowType.List type) {
-          return -1;
-        }
-
-        @Override
-        public Integer visit(ArrowType.Map type) {
-          return -1;
-        }
-
-        @Override
-        public Integer visit(ArrowType.Bool type) {
-          return 1;
-        }
-
-        @Override
-        public Integer visit(ArrowType.Int type) {
-          return type.getBitWidth() / 8;
-        }
-
-        @Override
-        public Integer visit(ArrowType.FloatingPoint type) {
-          switch (type.getPrecision()) {
-            case SINGLE:
-              return 4;
-            case DOUBLE:
-              return 8;
-            default:
-              return unsupported(type);
-          }
-        }
-
-        @Override
-        public Integer visit(ArrowType.Date type) {
-          return 4;
-        }
-
-        @Override
-        public Integer visit(ArrowType.Timestamp type) {
-          return 8;
-        }
-
-        @Override
-        public Integer visit(ArrowType.Binary type) {
-          return -1;
-        }
-
-        @Override
-        public Integer visit(ArrowType.Decimal type) {
-          return -1;
-        }
-
-        @Override
-        public Integer visit(ArrowType.Utf8 type) {
-          return -1;
-        }
-      };
-
-  private static final ArrowType.ArrowTypeVisitor<Type> typeIdVisitor =
-      new DefaultTypeVisitor<Type>() {
-
-        @Override
-        public Type visit(ArrowType.Bool type) {
-          return Type.BOOL;
-        }
-
-        @Override
-        public Type visit(ArrowType.Int type) {
-          if (type.getIsSigned()) {
-            int byteWidth = type.getBitWidth() / 8;
-            switch (byteWidth) {
-              case 1:
-                return Type.INT8;
-              case 2:
-                return Type.INT16;
-              case 4:
-                return Type.INT32;
-              case 8:
-                return Type.INT64;
-              default:
-                return unsupported(type);
+            @Override
+            public Integer visit(org.apache.arrow.vector.types.pojo.ArrowType.Struct type) {
+              return -1;
             }
-          }
-          return unsupported(type);
-        }
 
-        @Override
-        public Type visit(ArrowType.FloatingPoint type) {
-          switch (type.getPrecision()) {
-            case SINGLE:
-              return Type.FLOAT;
-            case DOUBLE:
-              return Type.DOUBLE;
-            default:
+            @Override
+            public Integer visit(org.apache.arrow.vector.types.pojo.ArrowType.List type) {
+              return -1;
+            }
+
+            @Override
+            public Integer visit(org.apache.arrow.vector.types.pojo.ArrowType.Map type) {
+              return -1;
+            }
+
+            @Override
+            public Integer visit(org.apache.arrow.vector.types.pojo.ArrowType.Bool type) {
+              return 1;
+            }
+
+            @Override
+            public Integer visit(org.apache.arrow.vector.types.pojo.ArrowType.Int type) {
+              return type.getBitWidth() / 8;
+            }
+
+            @Override
+            public Integer visit(org.apache.arrow.vector.types.pojo.ArrowType.FloatingPoint type) {
+              switch (type.getPrecision()) {
+                case SINGLE:
+                  return 4;
+                case DOUBLE:
+                  return 8;
+                default:
+                  return unsupported(type);
+              }
+            }
+
+            @Override
+            public Integer visit(org.apache.arrow.vector.types.pojo.ArrowType.Date type) {
+              return 4;
+            }
+
+            @Override
+            public Integer visit(org.apache.arrow.vector.types.pojo.ArrowType.Timestamp type) {
+              return 8;
+            }
+
+            @Override
+            public Integer visit(org.apache.arrow.vector.types.pojo.ArrowType.Binary type) {
+              return -1;
+            }
+
+            @Override
+            public Integer visit(org.apache.arrow.vector.types.pojo.ArrowType.Decimal type) {
+              return -1;
+            }
+
+            @Override
+            public Integer visit(org.apache.arrow.vector.types.pojo.ArrowType.Utf8 type) {
+              return -1;
+            }
+          };
+
+  private static final org.apache.arrow.vector.types.pojo.ArrowType.ArrowTypeVisitor<ArrowType>
+      typeIdVisitor =
+          new DefaultTypeVisitor<ArrowType>() {
+
+            @Override
+            public ArrowType visit(org.apache.arrow.vector.types.pojo.ArrowType.Bool type) {
+              return ArrowType.BOOL;
+            }
+
+            @Override
+            public ArrowType visit(org.apache.arrow.vector.types.pojo.ArrowType.Int type) {
+              if (type.getIsSigned()) {
+                int byteWidth = type.getBitWidth() / 8;
+                switch (byteWidth) {
+                  case 1:
+                    return ArrowType.INT8;
+                  case 2:
+                    return ArrowType.INT16;
+                  case 4:
+                    return ArrowType.INT32;
+                  case 8:
+                    return ArrowType.INT64;
+                  default:
+                    return unsupported(type);
+                }
+              }
               return unsupported(type);
-          }
-        }
+            }
 
-        @Override
-        public Type visit(ArrowType.Date type) {
-          switch (type.getUnit()) {
-            case DAY:
-              return Type.DATE32;
-            case MILLISECOND:
-              return Type.DATE64;
-            default:
-              return unsupported(type);
-          }
-        }
+            @Override
+            public ArrowType visit(
+                org.apache.arrow.vector.types.pojo.ArrowType.FloatingPoint type) {
+              switch (type.getPrecision()) {
+                case SINGLE:
+                  return ArrowType.FLOAT;
+                case DOUBLE:
+                  return ArrowType.DOUBLE;
+                default:
+                  return unsupported(type);
+              }
+            }
 
-        @Override
-        public Type visit(ArrowType.Timestamp type) {
-          return Type.TIMESTAMP;
-        }
+            @Override
+            public ArrowType visit(org.apache.arrow.vector.types.pojo.ArrowType.Date type) {
+              switch (type.getUnit()) {
+                case DAY:
+                  return ArrowType.DATE32;
+                case MILLISECOND:
+                  return ArrowType.DATE64;
+                default:
+                  return unsupported(type);
+              }
+            }
 
-        @Override
-        public Type visit(ArrowType.Binary type) {
-          return Type.BINARY;
-        }
+            @Override
+            public ArrowType visit(org.apache.arrow.vector.types.pojo.ArrowType.Timestamp type) {
+              return ArrowType.TIMESTAMP;
+            }
 
-        @Override
-        public Type visit(ArrowType.Decimal type) {
-          return Type.DECIMAL;
-        }
+            @Override
+            public ArrowType visit(org.apache.arrow.vector.types.pojo.ArrowType.Binary type) {
+              return ArrowType.BINARY;
+            }
 
-        @Override
-        public Type visit(ArrowType.Utf8 type) {
-          return Type.STRING;
-        }
+            @Override
+            public ArrowType visit(org.apache.arrow.vector.types.pojo.ArrowType.Decimal type) {
+              return ArrowType.DECIMAL;
+            }
 
-        @Override
-        public Type visit(ArrowType.Struct type) {
-          return Type.STRUCT;
-        }
+            @Override
+            public ArrowType visit(org.apache.arrow.vector.types.pojo.ArrowType.Utf8 type) {
+              return ArrowType.STRING;
+            }
 
-        @Override
-        public Type visit(ArrowType.List type) {
-          return Type.LIST;
-        }
+            @Override
+            public ArrowType visit(org.apache.arrow.vector.types.pojo.ArrowType.Struct type) {
+              return ArrowType.STRUCT;
+            }
 
-        @Override
-        public Type visit(ArrowType.Map type) {
-          return Type.MAP;
-        }
-      };
+            @Override
+            public ArrowType visit(org.apache.arrow.vector.types.pojo.ArrowType.List type) {
+              return ArrowType.LIST;
+            }
 
-  public static int getTypeWidth(ArrowType type) {
+            @Override
+            public ArrowType visit(org.apache.arrow.vector.types.pojo.ArrowType.Map type) {
+              return ArrowType.MAP;
+            }
+          };
+
+  public static int getTypeWidth(org.apache.arrow.vector.types.pojo.ArrowType type) {
     return type.accept(typeWidthVisitor);
   }
 
-  public static Type getTypeId(ArrowType type) {
+  public static ArrowType getTypeId(org.apache.arrow.vector.types.pojo.ArrowType type) {
     return type.accept(typeIdVisitor);
   }
 
-  public static short getTypeIdValue(ArrowType type) {
+  public static short getTypeIdValue(org.apache.arrow.vector.types.pojo.ArrowType type) {
     return type.accept(typeIdVisitor).getId();
   }
 
-  public static ArrowType.Bool bool() {
-    return ArrowType.Bool.INSTANCE;
+  public static org.apache.arrow.vector.types.pojo.ArrowType.Bool bool() {
+    return org.apache.arrow.vector.types.pojo.ArrowType.Bool.INSTANCE;
   }
 
-  public static ArrowType.Int intType(int bitWidth) {
-    return new ArrowType.Int(bitWidth, true);
+  public static org.apache.arrow.vector.types.pojo.ArrowType.Int intType(int bitWidth) {
+    return new org.apache.arrow.vector.types.pojo.ArrowType.Int(bitWidth, true);
   }
 
-  public static ArrowType.Int int8() {
+  public static org.apache.arrow.vector.types.pojo.ArrowType.Int int8() {
     return intType(8);
   }
 
-  public static ArrowType.Int int16() {
+  public static org.apache.arrow.vector.types.pojo.ArrowType.Int int16() {
     return intType(16);
   }
 
-  public static ArrowType.Int int32() {
+  public static org.apache.arrow.vector.types.pojo.ArrowType.Int int32() {
     return intType(32);
   }
 
-  public static ArrowType.Int int64() {
+  public static org.apache.arrow.vector.types.pojo.ArrowType.Int int64() {
     return intType(64);
   }
 
-  public static ArrowType.FloatingPoint float32() {
-    return new ArrowType.FloatingPoint(FloatingPointPrecision.SINGLE);
+  public static org.apache.arrow.vector.types.pojo.ArrowType.FloatingPoint float32() {
+    return new org.apache.arrow.vector.types.pojo.ArrowType.FloatingPoint(
+        FloatingPointPrecision.SINGLE);
   }
 
-  public static ArrowType.FloatingPoint float64() {
-    return new ArrowType.FloatingPoint(FloatingPointPrecision.DOUBLE);
+  public static org.apache.arrow.vector.types.pojo.ArrowType.FloatingPoint float64() {
+    return new org.apache.arrow.vector.types.pojo.ArrowType.FloatingPoint(
+        FloatingPointPrecision.DOUBLE);
   }
 
-  public static ArrowType.Date date32() {
-    return new ArrowType.Date(DateUnit.DAY);
+  public static org.apache.arrow.vector.types.pojo.ArrowType.Date date32() {
+    return new org.apache.arrow.vector.types.pojo.ArrowType.Date(DateUnit.DAY);
   }
 
-  public static ArrowType.Date date64() {
-    return new ArrowType.Date(DateUnit.MILLISECOND);
+  public static org.apache.arrow.vector.types.pojo.ArrowType.Date date64() {
+    return new org.apache.arrow.vector.types.pojo.ArrowType.Date(DateUnit.MILLISECOND);
   }
 
-  public static ArrowType.Timestamp timestamp() {
-    return new ArrowType.Timestamp(TimeUnit.MICROSECOND, null);
+  public static org.apache.arrow.vector.types.pojo.ArrowType.Timestamp timestamp() {
+    return new org.apache.arrow.vector.types.pojo.ArrowType.Timestamp(TimeUnit.MICROSECOND, null);
   }
 
-  public static ArrowType.Binary binary() {
-    return ArrowType.Binary.INSTANCE;
+  public static org.apache.arrow.vector.types.pojo.ArrowType.Binary binary() {
+    return org.apache.arrow.vector.types.pojo.ArrowType.Binary.INSTANCE;
   }
 
-  public static ArrowType.Utf8 utf8() {
-    return ArrowType.Utf8.INSTANCE;
+  public static org.apache.arrow.vector.types.pojo.ArrowType.Utf8 utf8() {
+    return org.apache.arrow.vector.types.pojo.ArrowType.Utf8.INSTANCE;
   }
 
-  public static ArrowType.Decimal decimal() {
+  public static org.apache.arrow.vector.types.pojo.ArrowType.Decimal decimal() {
     return decimal(DecimalUtils.MAX_PRECISION, DecimalUtils.MAX_SCALE);
   }
 
-  public static ArrowType.Decimal decimal(int precision, int scale) {
-    return new ArrowType.Decimal(precision, scale);
+  public static org.apache.arrow.vector.types.pojo.ArrowType.Decimal decimal(
+      int precision, int scale) {
+    return new org.apache.arrow.vector.types.pojo.ArrowType.Decimal(precision, scale);
   }
 
-  public static ArrowType.Decimal bigintDecimal() {
+  public static org.apache.arrow.vector.types.pojo.ArrowType.Decimal bigintDecimal() {
     return decimal(DecimalUtils.MAX_PRECISION, 0);
   }
 
@@ -297,7 +306,11 @@ public class DataTypes {
     return field(name, fieldType, Collections.emptyList());
   }
 
-  public static Field field(String name, boolean nullable, ArrowType type, Field... children) {
+  public static Field field(
+      String name,
+      boolean nullable,
+      org.apache.arrow.vector.types.pojo.ArrowType type,
+      Field... children) {
     return field(name, new FieldType(nullable, type, null), children);
   }
 
@@ -305,11 +318,16 @@ public class DataTypes {
     return field(name, fieldType, Arrays.asList(children));
   }
 
-  public static Field field(String name, boolean nullable, ArrowType type, List<Field> children) {
+  public static Field field(
+      String name,
+      boolean nullable,
+      org.apache.arrow.vector.types.pojo.ArrowType type,
+      List<Field> children) {
     return field(name, new FieldType(nullable, type, null), children);
   }
 
-  public static Field field(String name, ArrowType type, Field... children) {
+  public static Field field(
+      String name, org.apache.arrow.vector.types.pojo.ArrowType type, Field... children) {
     return field(name, true, type, children);
   }
 
@@ -317,34 +335,36 @@ public class DataTypes {
     return new Field(name, fieldType, children);
   }
 
-  public static Field notNullField(String name, ArrowType type, Field... children) {
+  public static Field notNullField(
+      String name, org.apache.arrow.vector.types.pojo.ArrowType type, Field... children) {
     return field(name, false, type, children);
   }
 
-  public static FieldType notNullFieldType(ArrowType type) {
+  public static FieldType notNullFieldType(org.apache.arrow.vector.types.pojo.ArrowType type) {
     return new FieldType(false, type, null);
   }
 
   /* ========================= array field utils ========================= */
-  public static Field primitiveArrayField(ArrowType type) {
+  public static Field primitiveArrayField(org.apache.arrow.vector.types.pojo.ArrowType type) {
     return primitiveArrayField("", type);
   }
 
-  public static Field primitiveArrayField(String name, ArrowType type) {
+  public static Field primitiveArrayField(
+      String name, org.apache.arrow.vector.types.pojo.ArrowType type) {
     return field(
         name,
-        FieldType.nullable(ArrowType.List.INSTANCE),
+        FieldType.nullable(org.apache.arrow.vector.types.pojo.ArrowType.List.INSTANCE),
         Collections.singletonList(field(ARRAY_ITEM_NAME, false, type)));
   }
 
-  public static Field arrayField(ArrowType type) {
+  public static Field arrayField(org.apache.arrow.vector.types.pojo.ArrowType type) {
     return arrayField("", type);
   }
 
-  public static Field arrayField(String name, ArrowType type) {
+  public static Field arrayField(String name, org.apache.arrow.vector.types.pojo.ArrowType type) {
     return field(
         name,
-        FieldType.nullable(ArrowType.List.INSTANCE),
+        FieldType.nullable(org.apache.arrow.vector.types.pojo.ArrowType.List.INSTANCE),
         Collections.singletonList(field(ARRAY_ITEM_NAME, true, type)));
   }
 
@@ -355,7 +375,7 @@ public class DataTypes {
   public static Field arrayField(String name, FieldType valueType) {
     return field(
         name,
-        FieldType.nullable(ArrowType.List.INSTANCE),
+        FieldType.nullable(org.apache.arrow.vector.types.pojo.ArrowType.List.INSTANCE),
         Collections.singletonList(field(ARRAY_ITEM_NAME, valueType)));
   }
 
@@ -365,7 +385,9 @@ public class DataTypes {
 
   public static Field arrayField(String name, Field valueField) {
     return field(
-        name, FieldType.nullable(ArrowType.List.INSTANCE), Collections.singletonList(valueField));
+        name,
+        FieldType.nullable(org.apache.arrow.vector.types.pojo.ArrowType.List.INSTANCE),
+        Collections.singletonList(valueField));
   }
 
   public static Field arrayElementField(Field field) {
@@ -373,11 +395,16 @@ public class DataTypes {
   }
 
   /* ========================= map field utils start ========================= */
-  public static Field mapField(ArrowType keyType, ArrowType itemType) {
+  public static Field mapField(
+      org.apache.arrow.vector.types.pojo.ArrowType keyType,
+      org.apache.arrow.vector.types.pojo.ArrowType itemType) {
     return mapField("", keyType, itemType);
   }
 
-  public static Field mapField(String name, ArrowType keyType, ArrowType itemType) {
+  public static Field mapField(
+      String name,
+      org.apache.arrow.vector.types.pojo.ArrowType keyType,
+      org.apache.arrow.vector.types.pojo.ArrowType itemType) {
     return mapField(
         name,
         field(MapVector.KEY_NAME, false, keyType),
@@ -392,7 +419,8 @@ public class DataTypes {
     Preconditions.checkArgument(!keyField.isNullable(), "Map's keys must be non-nullable");
     // Map's key-item pairs must be non-nullable structs
     Field valueField = structField(false, keyField, itemField);
-    return field(name, true, new ArrowType.Map(false), valueField);
+    return field(
+        name, true, new org.apache.arrow.vector.types.pojo.ArrowType.Map(false), valueField);
   }
 
   public static Field keyFieldForMap(Field mapField) {
@@ -425,11 +453,13 @@ public class DataTypes {
   }
 
   public static Field structField(String name, boolean nullable, Field... fields) {
-    return field(name, nullable, ArrowType.Struct.INSTANCE, fields);
+    return field(
+        name, nullable, org.apache.arrow.vector.types.pojo.ArrowType.Struct.INSTANCE, fields);
   }
 
   public static Field structField(String name, boolean nullable, List<Field> fields) {
-    return field(name, nullable, ArrowType.Struct.INSTANCE, fields);
+    return field(
+        name, nullable, org.apache.arrow.vector.types.pojo.ArrowType.Struct.INSTANCE, fields);
   }
 
   /* ========================= struct field utils end ========================= */
@@ -475,7 +505,7 @@ public class DataTypes {
   }
 
   private static long computeHash(long hash, Field field) {
-    Type typeID = getTypeId(field.getType());
+    ArrowType typeID = getTypeId(field.getType());
     while (true) {
       try {
         hash = Math.addExact(Math.multiplyExact(hash, 31), (long) typeID.getId());
