@@ -18,11 +18,19 @@
  */
 
 import { Scope } from "./scope";
-import { getMeta } from "../meta";
-import { TypeDescription } from "../description";
 import Fury from "../fury";
 
-class BinaryReaderBuilder {
+class TypeMetaBuilder {
+  constructor(private fury: string) {
+
+  }
+
+  fromBytes(reader: string) {
+    return `${this.fury}.typeMeta.fromBytes(${reader})`;
+  }
+}
+
+export class BinaryReaderBuilder {
   constructor(private holder: string) {
 
   }
@@ -289,28 +297,79 @@ class ClassResolverBuilder {
     return `${this.holder}.getSerializerById(${id})`;
   }
 
-  getSerializerByTag(tag: string) {
-    return `${this.holder}.getSerializerByTag(${tag})`;
-  }
-
-  createTagWriter(tag: string) {
-    return `${this.holder}.createTagWriter("${tag}")`;
-  }
-
-  readTag(binaryReader: string) {
-    return `${this.holder}.readTag(${binaryReader})`;
+  getSerializerByName(name: string) {
+    return `${this.holder}.getSerializerByName("${name}")`;
   }
 
   getSerializerByData(v: string) {
     return `${this.holder}.readTag(${v})`;
   }
+
+  getTypeInfo(v: string) {
+    return `${this.holder}.getTypeInfo("${v}")`;
+  }
+}
+
+class TypeMetaResolverBuilder {
+  constructor(private holder: string) {
+
+  }
+
+  ownName() {
+    return this.holder;
+  }
+
+  writeTypeMeta(typeInfo: string, writer: string, bytes: string) {
+    return `${this.holder}.writeTypeMeta(${typeInfo}, ${writer}, ${bytes})`;
+  }
+
+  readTypeMeta(reader: string) {
+    return `${this.holder}.readTypeMeta(${reader})`;
+  }
+
+  genSerializerByTypeMetaRuntime(typeMeta: string, ns: string, typeName: string) {
+    return `${this.holder}.genSerializerByTypeMetaRuntime(${typeMeta}, ${ns}, ${typeName})`;
+  }
+}
+
+class MetaStringResolverBuilder {
+  constructor(private holder: string) {
+
+  }
+
+  ownName() {
+    return this.holder;
+  }
+
+  writeBytes(writer: string, bytes: string) {
+    return `${this.holder}.writeBytes(${writer}, ${bytes})`;
+  }
+
+  readTypeName(reader: string) {
+    return `${this.holder}.readTypeName(${reader})`;
+  }
+
+  readNamespace(reader: string) {
+    return `${this.holder}.readNamespace(${reader})`;
+  }
+
+  encodeNamespace(input: string) {
+    return `${this.holder}.encodeNamespace("${input}")`;
+  }
+
+  encodeTypeName(input: string) {
+    return `${this.holder}.encodeTypeName("${input}")`;
+  }
 }
 
 export class CodecBuilder {
   reader: BinaryReaderBuilder;
   writer: BinaryWriterBuilder;
+  typeMeta: TypeMetaBuilder; // Use the TypeMetaWrapper
   referenceResolver: ReferenceResolverBuilder;
   classResolver: ClassResolverBuilder;
+  typeMetaResolver: TypeMetaResolverBuilder;
+  metaStringResolver: MetaStringResolverBuilder;
 
   constructor(scope: Scope, public fury: Fury) {
     const br = scope.declareByName("br", "fury.binaryReader");
@@ -321,18 +380,9 @@ export class CodecBuilder {
     this.writer = new BinaryWriterBuilder(bw);
     this.classResolver = new ClassResolverBuilder(cr);
     this.referenceResolver = new ReferenceResolverBuilder(rr);
-  }
-
-  furyName() {
-    return "fury";
-  }
-
-  meta(description: TypeDescription) {
-    return getMeta(description, this.fury);
-  }
-
-  config() {
-    return this.fury.config;
+    this.typeMeta = new TypeMetaBuilder("fury"); // Initialize the TypeMetaWrapper
+    this.typeMetaResolver = new TypeMetaResolverBuilder("fury.typeMetaResolver");
+    this.metaStringResolver = new MetaStringResolverBuilder("fury.metaStringResolver");
   }
 
   static isReserved(key: string) {
@@ -368,7 +418,19 @@ export class CodecBuilder {
     return prop;
   }
 
+  getFuryName() {
+    return "fury";
+  }
+
   getExternal(key: string) {
     return `external.${key}`;
   }
+
+  getOptions(key: string) {
+    return `options.${key}`;
+  }
+
+  getTypeInfo() {
+    return "typeInfo";
+  }
 }
