@@ -15,12 +15,12 @@
 # specific language governing permissions and limitations
 # under the License.
 
+import array
 import itertools
 import os
 import pickle
-from weakref import WeakValueDictionary
-
 import typing
+from weakref import WeakValueDictionary
 
 import pyfury.lib.mmh3
 from pyfury.buffer import Buffer
@@ -38,77 +38,107 @@ try:
 except ImportError:
     np = None
 
-from pyfury._serializer import (  # noqa: F401 # pylint: disable=unused-import
+from pyfury._fury import (
+    NOT_NULL_INT64_FLAG,
     BufferObject,
-    BytesBufferObject,
-    Serializer,
-    CrossLanguageCompatibleSerializer,
-    BooleanSerializer,
-    ByteSerializer,
-    Int16Serializer,
-    Int32Serializer,
-    Int64Serializer,
-    FloatSerializer,
-    DoubleSerializer,
-    StringSerializer,
-    DateSerializer,
-    TimestampSerializer,
-    BytesSerializer,
-    PyArraySerializer,
-    Numpy1DArraySerializer,
-    CollectionSerializer,
-    ListSerializer,
-    TupleSerializer,
-    StringArraySerializer,
-    SetSerializer,
-    MapSerializer,
-    SubMapSerializer,
-    EnumSerializer,
-    SliceSerializer,
-    PickleSerializer,
-    NOT_NULL_PYINT_FLAG,
-    PickleStrongCacheStub,
-    PickleCacheStub,
-    PICKLE_STRONG_CACHE_CLASS_ID,
-    PICKLE_CACHE_CLASS_ID,
 )
 
-try:
-    from pyfury._serialization import ENABLE_FURY_CYTHON_SERIALIZATION
+_WINDOWS = os.name == "nt"
 
-    if ENABLE_FURY_CYTHON_SERIALIZATION:
-        from pyfury._serialization import (  # noqa: F401, F811
-            BufferObject,
-            BytesBufferObject,
-            Serializer,
-            CrossLanguageCompatibleSerializer,
-            BooleanSerializer,
-            ByteSerializer,
-            Int16Serializer,
-            Int32Serializer,
-            Int64Serializer,
-            FloatSerializer,
-            DoubleSerializer,
-            StringSerializer,
-            DateSerializer,
-            TimestampSerializer,
-            BytesSerializer,
-            PyArraySerializer,
-            Numpy1DArraySerializer,
-            CollectionSerializer,
-            ListSerializer,
-            TupleSerializer,
-            StringArraySerializer,
-            SetSerializer,
-            MapSerializer,
-            SubMapSerializer,
-            EnumSerializer,
-            SliceSerializer,
-            PickleSerializer,
-            PickleStrongCacheStub,
-            PickleCacheStub,
-        )
-except ImportError:
+from pyfury._serialization import ENABLE_FURY_CYTHON_SERIALIZATION
+
+if ENABLE_FURY_CYTHON_SERIALIZATION:
+    from pyfury._serialization import (  # noqa: F401, F811
+        Serializer,
+        CrossLanguageCompatibleSerializer,
+        BooleanSerializer,
+        ByteSerializer,
+        Int16Serializer,
+        Int32Serializer,
+        Int64Serializer,
+        Float32Serializer,
+        Float64Serializer,
+        StringSerializer,
+        DateSerializer,
+        TimestampSerializer,
+        CollectionSerializer,
+        ListSerializer,
+        TupleSerializer,
+        StringArraySerializer,
+        SetSerializer,
+        MapSerializer,
+        SubMapSerializer,
+        EnumSerializer,
+        SliceSerializer,
+    )
+else:
+    from pyfury._serializer import (  # noqa: F401 # pylint: disable=unused-import
+        Serializer,
+        CrossLanguageCompatibleSerializer,
+        BooleanSerializer,
+        ByteSerializer,
+        Int16Serializer,
+        Int32Serializer,
+        Int64Serializer,
+        Float32Serializer,
+        Float64Serializer,
+        StringSerializer,
+        DateSerializer,
+        TimestampSerializer,
+        CollectionSerializer,
+        ListSerializer,
+        TupleSerializer,
+        StringArraySerializer,
+        SetSerializer,
+        MapSerializer,
+        SubMapSerializer,
+        EnumSerializer,
+        SliceSerializer,
+    )
+
+from pyfury.type import (
+    Int16ArrayType,
+    Int32ArrayType,
+    Int64ArrayType,
+    Float32ArrayType,
+    Float64ArrayType,
+    BoolNDArrayType,
+    Int16NDArrayType,
+    Int32NDArrayType,
+    Int64NDArrayType,
+    Float32NDArrayType,
+    Float64NDArrayType,
+    TypeId,
+)
+
+
+class NoneSerializer(Serializer):
+    def __init__(self, fury):
+        super().__init__(fury, None)
+        self.need_to_write_ref = False
+
+    def xwrite(self, buffer, value):
+        raise NotImplementedError
+
+    def xread(self, buffer):
+        raise NotImplementedError
+
+    def write(self, buffer, value):
+        pass
+
+    def read(self, buffer):
+        return None
+
+
+class _PickleStub:
+    pass
+
+
+class PickleStrongCacheStub:
+    pass
+
+
+class PickleCacheStub:
     pass
 
 
@@ -141,19 +171,6 @@ class PickleStrongCacheSerializer(Serializer):
     def xread(self, buffer):
         raise NotImplementedError
 
-    @staticmethod
-    def new_classinfo(fury, clear_threshold: int = 1000):
-        from pyfury import ClassInfo
-
-        return ClassInfo(
-            PickleStrongCacheStub,
-            PICKLE_STRONG_CACHE_CLASS_ID,
-            serializer=PickleStrongCacheSerializer(
-                fury, clear_threshold=clear_threshold
-            ),
-            class_name_bytes=b"PickleStrongCacheStub",
-        )
-
 
 class PickleCacheSerializer(Serializer):
     __slots__ = "_cached", "_reverse_cached"
@@ -190,17 +207,6 @@ class PickleCacheSerializer(Serializer):
     def xread(self, buffer):
         raise NotImplementedError
 
-    @staticmethod
-    def new_classinfo(fury):
-        from pyfury import ClassInfo
-
-        return ClassInfo(
-            PickleCacheStub,
-            PICKLE_CACHE_CLASS_ID,
-            serializer=PickleCacheSerializer(fury),
-            class_name_bytes=b"PickleCacheStub",
-        )
-
 
 class PandasRangeIndexSerializer(Serializer):
     __slots__ = "_cached"
@@ -216,7 +222,7 @@ class PandasRangeIndexSerializer(Serializer):
         stop = value.stop
         step = value.step
         if type(start) is int:
-            buffer.write_int16(NOT_NULL_PYINT_FLAG)
+            buffer.write_int16(NOT_NULL_INT64_FLAG)
             buffer.write_varint64(start)
         else:
             if start is None:
@@ -225,7 +231,7 @@ class PandasRangeIndexSerializer(Serializer):
                 buffer.write_int8(NOT_NULL_VALUE_FLAG)
                 fury.serialize_nonref(buffer, start)
         if type(stop) is int:
-            buffer.write_int16(NOT_NULL_PYINT_FLAG)
+            buffer.write_int16(NOT_NULL_INT64_FLAG)
             buffer.write_varint64(stop)
         else:
             if stop is None:
@@ -234,7 +240,7 @@ class PandasRangeIndexSerializer(Serializer):
                 buffer.write_int8(NOT_NULL_VALUE_FLAG)
                 fury.serialize_nonref(buffer, stop)
         if type(step) is int:
-            buffer.write_int16(NOT_NULL_PYINT_FLAG)
+            buffer.write_int16(NOT_NULL_INT64_FLAG)
             buffer.write_varint64(step)
         else:
             if step is None:
@@ -284,6 +290,7 @@ class DataClassSerializer(Serializer):
         # This will get superclass type hints too.
         self._type_hints = typing.get_type_hints(clz)
         self._field_names = sorted(self._type_hints.keys())
+        self._has_slots = hasattr(clz, "__slots__")
         # TODO compute hash
         self._hash = len(self._field_names)
         self._generated_write_method = self._gen_write_method()
@@ -296,16 +303,21 @@ class DataClassSerializer(Serializer):
     def _gen_write_method(self):
         context = {}
         counter = itertools.count(0)
-        buffer, fury, value = "buffer", "fury", "value"
+        buffer, fury, value, value_dict = "buffer", "fury", "value", "value_dict"
         context[fury] = self.fury
         stmts = [
             f'"""write method for {self.type_}"""',
             f"{buffer}.write_int32({self._hash})",
         ]
+        if not self._has_slots:
+            stmts.append(f"{value_dict} = {value}.__dict__")
         for field_name in self._field_names:
             field_type = self._type_hints[field_name]
             field_value = f"field_value{next(counter)}"
-            stmts.append(f"{field_value} = {value}.{field_name}")
+            if not self._has_slots:
+                stmts.append(f"{field_value} = {value_dict}['{field_name}']")
+            else:
+                stmts.append(f"{field_value} = {value}.{field_name}")
             if field_type is bool:
                 stmts.extend(gen_write_nullable_basic_stmts(buffer, field_value, bool))
             elif field_type == int:
@@ -328,7 +340,13 @@ class DataClassSerializer(Serializer):
 
     def _gen_read_method(self):
         context = dict(_jit_context)
-        buffer, fury, obj_class, obj = "buffer", "fury", "obj_class", "obj"
+        buffer, fury, obj_class, obj, obj_dict = (
+            "buffer",
+            "fury",
+            "obj_class",
+            "obj",
+            "obj_dict",
+        )
         ref_resolver = "ref_resolver"
         context[fury] = self.fury
         context[obj_class] = self.type_
@@ -342,9 +360,14 @@ class DataClassSerializer(Serializer):
             f"""   raise ClassNotCompatibleError(
             "Hash read_hash is not consistent with {self._hash} for {self.type_}")""",
         ]
+        if not self._has_slots:
+            stmts.append(f"{obj_dict} = {obj}.__dict__")
 
         def set_action(value: str):
-            return f"{obj}.{field_name} = {value}"
+            if not self._has_slots:
+                return f"{obj_dict}['{field_name}'] = {value}"
+            else:
+                return f"{obj}.{field_name} = {value}"
 
         for field_name in self._field_names:
             field_type = self._type_hints[field_name]
@@ -396,3 +419,251 @@ class DataClassSerializer(Serializer):
 
     def xread(self, buffer):
         raise NotImplementedError
+
+
+# Use numpy array or python array module.
+typecode_dict = (
+    {
+        # use bytes serializer for byte array.
+        "h": (2, Int16ArrayType, TypeId.INT16_ARRAY),
+        "i": (4, Int32ArrayType, TypeId.INT32_ARRAY),
+        "l": (8, Int64ArrayType, TypeId.INT64_ARRAY),
+        "f": (4, Float32ArrayType, TypeId.FLOAT32_ARRAY),
+        "d": (8, Float64ArrayType, TypeId.FLOAT64_ARRAY),
+    }
+    if not _WINDOWS
+    else {
+        "h": (2, Int16ArrayType, TypeId.INT16_ARRAY),
+        "l": (4, Int32ArrayType, TypeId.INT32_ARRAY),
+        "q": (8, Int64ArrayType, TypeId.INT64_ARRAY),
+        "f": (4, Float32ArrayType, TypeId.FLOAT32_ARRAY),
+        "d": (8, Float64ArrayType, TypeId.FLOAT64_ARRAY),
+    }
+)
+
+typeid_code = (
+    {
+        TypeId.INT16_ARRAY: "h",
+        TypeId.INT32_ARRAY: "i",
+        TypeId.INT64_ARRAY: "l",
+        TypeId.FLOAT32_ARRAY: "f",
+        TypeId.FLOAT64_ARRAY: "d",
+    }
+    if not _WINDOWS
+    else {
+        TypeId.INT16_ARRAY: "h",
+        TypeId.INT32_ARRAY: "l",
+        TypeId.INT64_ARRAY: "q",
+        TypeId.FLOAT32_ARRAY: "f",
+        TypeId.FLOAT64_ARRAY: "d",
+    }
+)
+
+
+class PyArraySerializer(CrossLanguageCompatibleSerializer):
+    typecode_dict = typecode_dict
+    typecodearray_type = (
+        {
+            "h": Int16ArrayType,
+            "i": Int32ArrayType,
+            "l": Int64ArrayType,
+            "f": Float32ArrayType,
+            "d": Float64ArrayType,
+        }
+        if not _WINDOWS
+        else {
+            "h": Int16ArrayType,
+            "l": Int32ArrayType,
+            "q": Int64ArrayType,
+            "f": Float32ArrayType,
+            "d": Float64ArrayType,
+        }
+    )
+
+    def __init__(self, fury, ftype, type_id: str):
+        super().__init__(fury, ftype)
+        self.typecode = typeid_code[type_id]
+        self.itemsize, ftype, self.type_id = typecode_dict[self.typecode]
+
+    def xwrite(self, buffer, value):
+        assert value.itemsize == self.itemsize
+        view = memoryview(value)
+        assert view.format == self.typecode
+        assert view.itemsize == self.itemsize
+        assert view.c_contiguous  # TODO handle contiguous
+        nbytes = len(value) * self.itemsize
+        buffer.write_varuint32(nbytes)
+        buffer.write_buffer(value)
+
+    def xread(self, buffer):
+        data = buffer.read_bytes_and_size()
+        arr = array.array(self.typecode, [])
+        arr.frombytes(data)
+        return arr
+
+    def write(self, buffer, value: array.array):
+        nbytes = len(value) * value.itemsize
+        buffer.write_string(value.typecode)
+        buffer.write_varuint32(nbytes)
+        buffer.write_buffer(value)
+
+    def read(self, buffer):
+        typecode = buffer.read_string()
+        data = buffer.read_bytes_and_size()
+        arr = array.array(typecode, [])
+        arr.frombytes(data)
+        return arr
+
+
+class DynamicPyArraySerializer(Serializer):
+    def xwrite(self, buffer, value):
+        itemsize, ftype, type_id = typecode_dict[value.typecode]
+        view = memoryview(value)
+        nbytes = len(value) * itemsize
+        buffer.write_varuint32(type_id)
+        buffer.write_varuint32(nbytes)
+        if not view.c_contiguous:
+            buffer.write_bytes(value.tobytes())
+        else:
+            buffer.write_buffer(value)
+
+    def xread(self, buffer):
+        type_id = buffer.read_varint32()
+        typecode = typeid_code[type_id]
+        data = buffer.read_bytes_and_size()
+        arr = array.array(typecode, [])
+        arr.frombytes(data)
+        return arr
+
+    def write(self, buffer, value):
+        buffer.write_varuint32(PickleSerializer.PICKLE_CLASS_ID)
+        self.fury.handle_unsupported_write(buffer, value)
+
+    def read(self, buffer):
+        return self.fury.handle_unsupported_read(buffer)
+
+
+if np:
+    _np_dtypes_dict = (
+        {
+            # use bytes serializer for byte array.
+            np.dtype(np.bool_): (1, "?", BoolNDArrayType, TypeId.BOOL_ARRAY),
+            np.dtype(np.int16): (2, "h", Int16NDArrayType, TypeId.INT16_ARRAY),
+            np.dtype(np.int32): (4, "i", Int32NDArrayType, TypeId.INT32_ARRAY),
+            np.dtype(np.int64): (8, "l", Int64NDArrayType, TypeId.INT64_ARRAY),
+            np.dtype(np.float32): (4, "f", Float32NDArrayType, TypeId.FLOAT32_ARRAY),
+            np.dtype(np.float64): (8, "d", Float64NDArrayType, TypeId.FLOAT64_ARRAY),
+        }
+        if not _WINDOWS
+        else {
+            np.dtype(np.bool_): (1, "?", BoolNDArrayType, TypeId.BOOL_ARRAY),
+            np.dtype(np.int16): (2, "h", Int16NDArrayType, TypeId.INT16_ARRAY),
+            np.dtype(np.int32): (4, "l", Int32NDArrayType, TypeId.INT32_ARRAY),
+            np.dtype(np.int64): (8, "q", Int64NDArrayType, TypeId.INT64_ARRAY),
+            np.dtype(np.float32): (4, "f", Float32NDArrayType, TypeId.FLOAT32_ARRAY),
+            np.dtype(np.float64): (8, "d", Float64NDArrayType, TypeId.FLOAT64_ARRAY),
+        }
+    )
+else:
+    _np_dtypes_dict = {}
+
+
+class Numpy1DArraySerializer(Serializer):
+    dtypes_dict = _np_dtypes_dict
+
+    def __init__(self, fury, ftype, dtype):
+        super().__init__(fury, ftype)
+        self.dtype = dtype
+        self.itemsize, self.format, self.typecode, self.type_id = _np_dtypes_dict[
+            self.dtype
+        ]
+
+    def xwrite(self, buffer, value):
+        assert value.itemsize == self.itemsize
+        view = memoryview(value)
+        try:
+            assert view.format == self.typecode
+        except AssertionError as e:
+            raise e
+        assert view.itemsize == self.itemsize
+        nbytes = len(value) * self.itemsize
+        buffer.write_varuint32(nbytes)
+        if self.dtype == np.dtype("bool") or not view.c_contiguous:
+            buffer.write_bytes(value.tobytes())
+        else:
+            buffer.write_buffer(value)
+
+    def xread(self, buffer):
+        data = buffer.read_bytes_and_size()
+        return np.frombuffer(data, dtype=self.dtype)
+
+    def write(self, buffer, value):
+        buffer.write_int8(PickleSerializer.PICKLE_CLASS_ID)
+        self.fury.handle_unsupported_write(buffer, value)
+
+    def read(self, buffer):
+        return self.fury.handle_unsupported_read(buffer)
+
+
+class NDArraySerializer(Serializer):
+    def xwrite(self, buffer, value):
+        itemsize, typecode, ftype, type_id = _np_dtypes_dict[value.dtype]
+        view = memoryview(value)
+        nbytes = len(value) * itemsize
+        buffer.write_varuint32(type_id)
+        buffer.write_varuint32(nbytes)
+        if value.dtype == np.dtype("bool") or not view.c_contiguous:
+            buffer.write_bytes(value.tobytes())
+        else:
+            buffer.write_buffer(value)
+
+    def xread(self, buffer):
+        raise NotImplementedError("Multi-dimensional array not supported currently")
+
+    def write(self, buffer, value):
+        buffer.write_int8(PickleSerializer.PICKLE_CLASS_ID)
+        self.fury.handle_unsupported_write(buffer, value)
+
+    def read(self, buffer):
+        return self.fury.handle_unsupported_read(buffer)
+
+
+class BytesSerializer(CrossLanguageCompatibleSerializer):
+    def write(self, buffer, value):
+        self.fury.write_buffer_object(buffer, BytesBufferObject(value))
+
+    def read(self, buffer):
+        fury_buf = self.fury.read_buffer_object(buffer)
+        return fury_buf.to_pybytes()
+
+
+class BytesBufferObject(BufferObject):
+    __slots__ = ("binary",)
+
+    def __init__(self, binary: bytes):
+        self.binary = binary
+
+    def total_bytes(self) -> int:
+        return len(self.binary)
+
+    def write_to(self, buffer: "Buffer"):
+        buffer.write_bytes(self.binary)
+
+    def to_buffer(self) -> "Buffer":
+        return Buffer(self.binary)
+
+
+class PickleSerializer(Serializer):
+    PICKLE_CLASS_ID = 96
+
+    def xwrite(self, buffer, value):
+        raise NotImplementedError
+
+    def xread(self, buffer):
+        raise NotImplementedError
+
+    def write(self, buffer, value):
+        self.fury.handle_unsupported_write(buffer, value)
+
+    def read(self, buffer):
+        return self.fury.handle_unsupported_read(buffer)
