@@ -19,6 +19,7 @@ import array
 import datetime
 import gc
 import io
+import os
 import pickle
 import weakref
 from enum import Enum
@@ -41,7 +42,7 @@ from pyfury.serializer import (
     Numpy1DArraySerializer,
 )
 from pyfury.tests.core import require_pyarrow
-from pyfury.type import FuryType
+from pyfury.type import TypeId
 from pyfury.util import lazy_import
 
 pa = lazy_import("pyarrow")
@@ -52,7 +53,7 @@ def test_float():
     assert ser_de(fury, -1.0) == -1.0
     assert ser_de(fury, 1 / 3) == 1 / 3
     serializer = fury.class_resolver.get_serializer(float)
-    assert type(serializer) is pyfury.DoubleSerializer
+    assert type(serializer) is pyfury.Float64Serializer
 
 
 def test_tuple():
@@ -61,8 +62,18 @@ def test_tuple():
     assert ser_de(fury, (-1.0, 2)) == (-1.0, 2)
 
 
-def test_dict():
+def test_string():
     fury = Fury(language=Language.PYTHON, ref_tracking=True)
+    assert ser_de(fury, "hello") == "hello"
+    assert ser_de(fury, "hello，世界") == "hello，世界"
+    assert ser_de(fury, "hello，世界" * 10) == "hello，世界" * 10
+    assert ser_de(fury, "hello，😀") == "hello，😀"
+    assert ser_de(fury, "hello，😀" * 10) == "hello，😀" * 10
+
+
+@pytest.mark.parametrize("track_ref", [False, True])
+def test_dict(track_ref):
+    fury = Fury(language=Language.PYTHON, ref_tracking=track_ref)
     assert ser_de(fury, {1: 2}) == {1: 2}
     assert ser_de(fury, {1 / 3: 2.0}) == {1 / 3: 2.0}
     assert ser_de(fury, {1 / 3: 2}) == {1 / 3: 2}
@@ -82,17 +93,55 @@ def test_dict():
     assert ser_de(fury, dict3) == dict3
 
 
+@pytest.mark.parametrize("track_ref", [False, True])
+def test_multi_chunk_simple_dict(track_ref):
+    fury = Fury(language=Language.PYTHON, ref_tracking=track_ref)
+    dict0 = {
+        1: 2.0,
+        2: 3,
+        4.0: True,
+    }
+    assert ser_de(fury, dict0) == dict0
+
+
+@pytest.mark.parametrize("track_ref", [False, True])
+def test_multi_chunk_complex_dict(track_ref):
+    fury = Fury(language=Language.PYTHON, ref_tracking=track_ref)
+    now = datetime.datetime.now()
+    day = datetime.date(2021, 11, 23)
+    dict0 = {"a": "a", 1: 1, -1.0: -1.0, True: True, now: now, day: day}
+    assert ser_de(fury, dict0) == dict0
+
+
+@pytest.mark.parametrize("track_ref", [False, True])
+def test_big_chunk_dict(track_ref):
+    fury = Fury(language=Language.PYTHON, ref_tracking=track_ref)
+    now = datetime.datetime.now()
+    day = datetime.date(2021, 11, 23)
+    dict0 = {}
+    values = ["a", 1, -1.0, True, False, now, day]
+    for i in range(1000):
+        dict0[i] = values[i % len(values)]
+        dict0[f"key{i}"] = values[i % len(values)]
+        dict0[float(i)] = values[i % len(values)]
+    assert ser_de(fury, dict0) == dict0
+
+
 @pytest.mark.parametrize("language", [Language.XLANG, Language.PYTHON])
 def test_basic_serializer(language):
     fury = Fury(language=language, ref_tracking=True)
-    datetime_serializer = fury.class_resolver.get_serializer(datetime.datetime)
+    classinfo = fury.class_resolver.get_classinfo(datetime.datetime)
     assert isinstance(
-        datetime_serializer, (TimestampSerializer, _serialization.TimestampSerializer)
+        classinfo.serializer, (TimestampSerializer, _serialization.TimestampSerializer)
     )
-    assert datetime_serializer.get_xtype_id() == FuryType.TIMESTAMP.value
-    date_serializer = fury.class_resolver.get_serializer(datetime.date)
-    assert isinstance(date_serializer, (DateSerializer, _serialization.DateSerializer))
-    assert date_serializer.get_xtype_id() == FuryType.DATE32.value
+    if language == Language.XLANG:
+        assert classinfo.type_id == TypeId.TIMESTAMP
+    classinfo = fury.class_resolver.get_classinfo(datetime.date)
+    assert isinstance(
+        classinfo.serializer, (DateSerializer, _serialization.DateSerializer)
+    )
+    if language == Language.XLANG:
+        assert classinfo.type_id == TypeId.LOCAL_DATE
     assert ser_de(fury, True) is True
     assert ser_de(fury, False) is False
     assert ser_de(fury, -1) == -1
@@ -227,7 +276,8 @@ def test_array_serializer(language):
     fury = Fury(language=language, ref_tracking=True, require_class_registration=False)
     for typecode in PyArraySerializer.typecode_dict.keys():
         arr = array.array(typecode, list(range(10)))
-        assert ser_de(fury, arr) == arr
+        new_arr = ser_de(fury, arr)
+        assert np.array_equal(new_arr, arr)
     for dtype in Numpy1DArraySerializer.dtypes_dict.keys():
         arr = np.array(list(range(10)), dtype=dtype)
         new_arr = ser_de(fury, arr)
@@ -235,6 +285,28 @@ def test_array_serializer(language):
         np.testing.assert_array_equal(new_arr, arr)
 
 
+def test_numpy_array_memoryview():
+    _WINDOWS = os.name == "nt"
+    if _WINDOWS:
+        arr = np.array(list(range(10)), dtype="int32")
+        view = memoryview(arr)
+        assert view.format == "l"
+        assert view.itemsize == 4
+        arr = np.array(list(range(10)), dtype="int64")
+        view = memoryview(arr)
+        assert view.format == "q"
+        assert view.itemsize == 8
+    else:
+        arr = np.array(list(range(10)), dtype="int32")
+        view = memoryview(arr)
+        assert view.format == "i"
+        assert view.itemsize == 4
+        arr = np.array(list(range(10)), dtype="int64")
+        view = memoryview(arr)
+        assert view.format == "l"
+        assert view.itemsize == 8
+
+
 def ser_de(fury, obj):
     binary = fury.serialize(obj)
     return fury.deserialize(binary)
@@ -330,12 +402,6 @@ class BarSerializer(pyfury.Serializer):
     def xread(self, buffer):
         return Bar(buffer.read_int32(), buffer.read_int32())
 
-    def get_xtype_id(self):
-        return pyfury.FuryType.FURY_TYPE_TAG.value
-
-    def get_xtype_tag(self):
-        return "test.Bar"
-
 
 class RegisterClass:
     def __init__(self, f1=None):
@@ -362,7 +428,7 @@ def test_register_py_serializer():
         def xread(self, buffer):
             raise NotImplementedError
 
-    fury.register_serializer(A, Serializer(fury, RegisterClass))
+    fury.register_type(A, serializer=Serializer(fury, RegisterClass))
     assert fury.deserialize(fury.serialize(RegisterClass(100))).f1 == 100
 
 
@@ -372,7 +438,7 @@ class A:
             pass
 
 
-def test_register_class():
+def test_register_type():
     fury = Fury(language=Language.PYTHON, ref_tracking=True)
 
     class Serializer(pyfury.Serializer):
@@ -388,9 +454,9 @@ def test_register_class():
         def xread(self, buffer):
             raise NotImplementedError
 
-    fury.register_serializer(A, Serializer(fury, A))
-    fury.register_serializer(A.B, Serializer(fury, A.B))
-    fury.register_serializer(A.B.C, Serializer(fury, A.B.C))
+    fury.register_type(A, serializer=Serializer(fury, A))
+    fury.register_type(A.B, serializer=Serializer(fury, A.B))
+    fury.register_type(A.B.C, serializer=Serializer(fury, A.B.C))
     assert isinstance(fury.deserialize(fury.serialize(A())), A)
     assert isinstance(fury.deserialize(fury.serialize(A.B())), A.B)
     assert isinstance(fury.deserialize(fury.serialize(A.B.C())), A.B.C)
@@ -411,7 +477,9 @@ def test_pickle_fallback():
 
 
 def test_unsupported_callback():
-    fury = Fury(language=Language.PYTHON, ref_tracking=True)
+    fury = Fury(
+        language=Language.PYTHON, ref_tracking=True, require_class_registration=False
+    )
 
     def f1(x):
         return x
@@ -420,8 +488,6 @@ def test_unsupported_callback():
         return x + x
 
     obj1 = [1, True, f1, f2, {1: 2}]
-    with pytest.raises(Exception):
-        fury.serialize(obj1)
     unsupported_objects = []
     binary1 = fury.serialize(obj1, unsupported_callback=unsupported_objects.append)
     assert len(unsupported_objects) == 2
@@ -469,12 +535,6 @@ def test_enum():
     assert ser_de(fury, EnumClass.E3) == EnumClass.E3
     assert ser_de(fury, EnumClass.E4) == EnumClass.E4
     assert isinstance(fury.class_resolver.get_serializer(EnumClass), EnumSerializer)
-    assert isinstance(
-        fury.class_resolver.get_serializer(obj=EnumClass.E1), EnumSerializer
-    )
-    assert isinstance(
-        fury.class_resolver.get_serializer(obj=EnumClass.E4), EnumSerializer
-    )
 
 
 def test_duplicate_serialize():
@@ -494,25 +554,19 @@ class CacheClass1:
 
 def test_cache_serializer():
     fury = Fury(language=Language.PYTHON, ref_tracking=True)
-    fury.register_serializer(CacheClass1, pyfury.PickleStrongCacheSerializer(fury))
+    fury.register_type(CacheClass1, serializer=pyfury.PickleStrongCacheSerializer(fury))
     assert ser_de(fury, CacheClass1(1)) == CacheClass1(1)
-    fury.register_serializer(CacheClass1, pyfury.PickleCacheSerializer(fury))
+    fury.register_type(CacheClass1, serializer=pyfury.PickleCacheSerializer(fury))
     assert ser_de(fury, CacheClass1(1)) == CacheClass1(1)
 
-    classinfo = pyfury.PickleStrongCacheSerializer.new_classinfo(fury)
-    buffer = Buffer.allocate(32)
-    fury.serialize_ref(buffer, CacheClass1(1), classinfo)
-    assert fury.deserialize_ref(buffer) == CacheClass1(1)
-    classinfo = pyfury.PickleCacheSerializer.new_classinfo(fury)
-    fury.serialize_ref(buffer, CacheClass1(1), classinfo)
-    assert fury.deserialize_ref(buffer) == CacheClass1(1)
-
 
 def test_pandas_range_index():
     fury = Fury(
         language=Language.PYTHON, ref_tracking=True, require_class_registration=False
     )
-    fury.register_serializer(pd.RangeIndex, pyfury.PandasRangeIndexSerializer(fury))
+    fury.register_type(
+        pd.RangeIndex, serializer=pyfury.PandasRangeIndexSerializer(fury)
+    )
     index = pd.RangeIndex(1, 100, 2, name="a")
     new_index = ser_de(fury, index)
     pd.testing.assert_index_equal(new_index, new_index)
@@ -530,7 +584,9 @@ class PyDataClass1:
 
 
 def test_py_serialize_dataclass():
-    fury = Fury(language=Language.PYTHON, ref_tracking=True)
+    fury = Fury(
+        language=Language.PYTHON, ref_tracking=True, require_class_registration=False
+    )
     obj1 = PyDataClass1(
         f1=1, f2=-2.0, f3="abc", f4=True, f5="xyz", f6=[1, 2], f7={"k1": "v1"}
     )
@@ -555,3 +611,35 @@ def test_function():
     df = pd.DataFrame({"a": list(range(10))})
     df_sum = fury.deserialize(fury.serialize(df.sum))
     assert df_sum().equals(df.sum())
+
+
+@dataclass(unsafe_hash=True)
+class MapFields:
+    simple_dict: dict = None
+    empty_dict: dict = None
+    large_dict: dict = None
+
+
+def test_map_fields_chunk_serializer():
+    fury = Fury(
+        language=Language.PYTHON, ref_tracking=True, require_class_registration=False
+    )
+
+    simple_dict = {"a": 1, "b": 2, "c": 3}
+    empty_dict = {}
+    large_dict = {f"key{i}": i for i in range(1000)}
+
+    # MapSerializer test
+    map_fields_object = MapFields(
+        simple_dict=simple_dict, empty_dict=empty_dict, large_dict=large_dict
+    )
+
+    serialized = fury.serialize(map_fields_object)
+    deserialized = fury.deserialize(serialized)
+    assert map_fields_object.simple_dict == deserialized.simple_dict
+    assert map_fields_object.empty_dict == deserialized.empty_dict
+    assert map_fields_object.large_dict == deserialized.large_dict
+
+
+if __name__ == "__main__":
+    test_string()
