@@ -26,8 +26,7 @@ from os.path import abspath
 from os.path import join as pjoin
 
 import setuptools
-from setuptools import find_packages  # Must import before Cython
-from distutils.core import setup
+from setuptools import find_packages, setup  # Must import before Cython
 import Cython
 import numpy as np
 import pyarrow as pa
@@ -38,7 +37,7 @@ try:
 except FileExistsError:
     pass
 
-pyarrow_version = "12.0.0"
+pyarrow_version = "15.0.0" if sys.version_info.minor < 13 else "18.0.0"
 # Check if we're running 64-bit Python
 if not sys.maxsize > 2**32:
     raise RuntimeError("Not supported on 32-bit")
@@ -115,6 +114,7 @@ _pkg_files = [
     "*.so",
     "*.dylib",
     "*.dll",
+    "*.pyd",
 ]
 
 
